define([
    'app',

    'config/angular',
    'config/angularAgility',
    'config/auth',
    'config/dataLayer',
    'config/facebook',
    'config/framework',
    'config/google',
    'config/http',
    'config/moment',

    'run/globalMethods',
    'run/onRun',
    'run/userEvents',

    'filters/bindName',
    'filters/distance',
    'filters/i18n',
    'filters/itemPrice',
    'filters/enhancedCurrency',
    'filters/timeRange',

    'routes/addressRoutes',
    'routes/advertRoutes',
    'routes/categoryRoutes',
    'routes/favouriteRoutes',
    'routes/homeRoutes',
    'routes/indexRoutes',
    'routes/locationRoutes',
    'routes/loyaltyRoutes',
    'routes/merchantRoutes',
    'routes/orderRoutes',
    'routes/orderStepsRoutes',
    'routes/otherRoutes',
    'routes/paymentRoutes',
    'routes/profileRoutes',
    'routes/userRoutes',
    'routes/vouchercodeRoutes',

    'resolvers/addressResolver',
    'resolvers/advertResolver',
    'resolvers/categoryResolver',
    'resolvers/merchantResolver',
    'resolvers/orderResolver',
    'resolvers/locationResolver',
    'resolvers/loyaltyResolver',
    'resolvers/paymentResolver',
    'resolvers/userResolver',
    'resolvers/vouchercodeResolver',

    'resources/config_constants',
    'resources/country_options',
    'resources/order_action_reason_options',
    'resources/order_approval_status_options',
    'resources/payment_method_options',
    'resources/pre_total_gross',
    'resources/pre_total_net',
    'resources/radius_options',
    'resources/response_enum',
    'resources/sorting_options',
    'resources/state_options',
    'resources/texts',

    'services/addressService',
    'services/advertService',
    'services/categoryService',
    'services/frameworkService',
    'services/favouriteService',
    'services/fencingService',
    'services/geolocationService',
    'services/headerTitleService',
    'services/keenService',
    'services/leftMenuService',
    'services/localStorageService',
    'services/locationService',
    'services/loyaltyService',
    'services/menuService',
    'services/merchantService',
    'services/orderingUtilService',
    'services/orderService',
    'services/paymentService',
    'services/notificationService',
    'services/restService',
    'services/settingsService',
    'services/surveyService',
    'services/userService',
    'services/utilityService',
    'services/vouchercodeService',
    'services/taggingService',

    'directives/addressMap',
    'directives/advertFavourite',
    'directives/confirm',
    'directives/countdown',
    'directives/enhancedSwitchWhen',
    'directives/equalTo',
    'directives/fadeIn',
    'directives/favourite',
    'directives/focusOnError',
    'directives/integerCurrency',
    'directives/loginPrompt',
    'directives/menuItems',
    'directives/openingHours',
    'directives/passwordPolicy',
    'directives/passwordPrompt',
    'directives/share',
    'directives/vouchercode',

    'controllers/advert/advertDetailController',
    'controllers/advert/advertListController',
    'controllers/category/categoryListController',
    'controllers/location/locationDetailController',
    'controllers/location/locationListController',
    'controllers/location/locationListGroupedByMerchantController',
    'controllers/loyalty/rewardDetailController',
    'controllers/merchant/merchantListController',
    'controllers/order/cartController',
    'controllers/order/discountsController',
    'controllers/order/itemDetailController',
    'controllers/order/menuListController',
    'controllers/order/orderCheckoutController',
    'controllers/order/orderDetailController',
    'controllers/order/orderLocationListController',
    'controllers/order/orderPromocodeController',
    'controllers/other/faqsController',
    'controllers/other/termsController',
    'controllers/other/disclaimersController',
    'controllers/other/instructionsController',
    'controllers/other/fencingController',
    'controllers/other/headerTitleController',
    'controllers/other/iframeController',
    'controllers/other/leftMenuController',
    'controllers/other/rightMenuController',
    'controllers/other/settingsController',
    'controllers/payment/paymentDetailController',
    'controllers/payment/paymentMethodAddController',
    'controllers/payment/paymentTokenController',
    'controllers/user/activityController',
    'controllers/user/addressEditController',
    'controllers/user/changePasswordController',
    'controllers/user/forgotPasswordController',
    'controllers/user/loginController',
    'controllers/user/logoutController',
    'controllers/user/profileController',
    'controllers/user/referFriendController',
    'controllers/user/resetPasswordController',
    'controllers/user/userActivateController',
    'controllers/user/userEditController',
    'controllers/user/userUpdateUsernameController',
    'controllers/vouchercode/vouchercodeListByUserController',
    'controllers/vouchercode/vouchercodeDetailController',
    'controllers/indexController'
], function (
    app,

    angular,
    angularAgility,
    auth,
    dataLayer,
    facebook,
    framework,
    google,
    http,
    moment,

    globalMethods,
    onRun,
    userEvents,

    bindNameFilter,
    distanceFilter,
    i18nFilter,
    itemPriceFilter,
    enhancedCurrencyFilter,
    timeRangeFilter,

    addressRoutes,
    advertRoutes,
    categoryRoutes,
    favouriteRoutes,
    homeRoutes,
    indexRoutes,
    locationRoutes,
    loyaltyRoutes,
    merchantRoutes,
    orderRoutes,
    orderStepsRoutes,
    otherRoutes,
    paymentRoutes,
    profileRoutes,
    userRoutes,
    vouchercodeRoutes,

    addressResolver,
    advertResolver,
    categoryResolver,
    merchantResolver,
    orderResolver,
    locationResolver,
    loyaltyResolver,
    paymentResolver,
    userResolver,
    vouchercodeResolver,

    config_constants,
    country_options,
    order_action_reason_options,
    order_approval_status_options,
    payment_method_options,
    pre_total_gross,
    pre_total_net,
    radius_options,
    response_enum,
    sorting_options,
    state_options,
    texts,

    addressService,
    advertService,
    categoryService,
    frameworkService,
    favouriteService,
    fencingService,
    geolocationService,
    headerTitleService,
    keenService,
    leftMenuService,
    localStorageService,
    locationService,
    loyaltyService,
    menuService,
    merchantService,
    orderingUtilService,
    orderService,
    paymentService,
    notificationService,
    restService,
    settingsService,
    surveyService,
    userService,
    utilityService,
    vouchercodeService,
    taggingService,

    addressMap,
    advertFavourite,
    confirm,
    countdown,
    enhancedSwitchWhen,
    equalTo,
    fadeIn,
    favourite,
    focusOnError,
    integerCurrency,
    loginPrompt,
    menuItems,
    openingHours,
    passwordPolicy,
    passwordPrompt,
    share,
    vouchercode,

    advertDetailController,
    advertListController,
    categoryListController,
    locationDetailController,
    locationListController,
    locationListGroupedByMerchantController,
    rewardDetailController,
    merchantListController,
    cartController,
    discountsController,
    itemDetailController,
    menuListController,
    orderCheckoutController,
    orderDetailController,
    orderLocationListController,
    orderPromocodeController,
    faqsController,
    termsController,
    disclaimersController,
    instructionsController,
    fencingController,
    headerTitleController,
    iframeController,
    leftMenuController,
    rightMenuController,
    settingsController,
    paymentDetailController,
    paymentMethodAddController,
    paymentTokenController,
    activityController,
    addressEditController,
    changePasswordController,
    forgotPasswordController,
    loginController,
    logoutController,
    profileController,
    referFriendController,
    resetPasswordController,
    userActivateController,
    userEditController,
    userUpdateUsernameController,
    vouchercodeListByUserController,
    vouchercodeDetailController,
    indexController
) {
        app.config(addressRoutes);
        app.config(advertRoutes);
        app.config(categoryRoutes);
        app.config(favouriteRoutes);
        app.config(homeRoutes);
        app.config(indexRoutes);
        app.config(locationRoutes);
        app.config(merchantRoutes);
        app.config(otherRoutes);
        app.config(paymentRoutes);
        app.config(profileRoutes);
        app.config(userRoutes);
        app.config(vouchercodeRoutes);

        app.filter('bindName', bindNameFilter);
        app.filter('distance', distanceFilter);
        app.filter('i18n', i18nFilter);
        app.filter('itemPrice', itemPriceFilter);
        app.filter('enhancedCurrency', enhancedCurrencyFilter);
        app.filter('timeRange', timeRangeFilter);

        app.service('addressResolver', addressResolver);
        app.service('advertResolver', advertResolver);
        app.service('categoryResolver', categoryResolver);
        app.service('merchantResolver', merchantResolver);
        app.service('locationResolver', locationResolver);
        app.service('paymentResolver', paymentResolver);
        app.service('userResolver', userResolver);
        app.service('vouchercodeResolver', vouchercodeResolver);

        app.constant('config_constants', config_constants);
        app.constant('country_options', country_options);
        app.constant('payment_method_options', payment_method_options);
        app.constant('radius_options', radius_options);
        app.constant('response_enum', response_enum);
        app.constant('sorting_options', sorting_options);
        app.constant('state_options', state_options);
        app.constant('texts', texts);

        app.service('addressService', addressService);
        app.service('advertService', advertService);
        app.service('categoryService', categoryService);
        app.service('frameworkService', frameworkService);
        app.service('favouriteService', favouriteService);
        app.service('fencingService', fencingService);
        app.service('geolocationService', geolocationService);
        app.service('headerTitleService', headerTitleService);
        app.service('keenService', keenService);
        app.service('leftMenuService', leftMenuService);
        app.service('localStorageService', localStorageService);
        app.service('locationService', locationService);
        app.service('merchantService', merchantService);
        app.service('paymentService', paymentService);
        app.service('notificationService', notificationService);
        app.service('restService', restService);
        app.service('settingsService', settingsService);
        app.service('surveyService', surveyService);
        app.service('userService', userService);
        app.service('utilityService', utilityService);
        app.service('vouchercodeService', vouchercodeService);
        app.service('taggingService', taggingService);

        app.directive('addressMap', addressMap);
        app.directive('advertFavourite', advertFavourite);
        app.directive('confirm', confirm);
        app.directive('countdown', countdown);
        app.directive('enhancedSwitchWhen', enhancedSwitchWhen);
        app.directive('equalTo', equalTo);
        app.directive('fadeIn', fadeIn);
        app.directive('favourite', favourite);
        app.directive('focusOnError', focusOnError);
        app.directive('integerCurrency', integerCurrency);
        app.directive('loginPrompt', loginPrompt);
        app.directive('menuItems', menuItems);
        app.directive('openingHours', openingHours);
        app.directive('passwordPolicy', passwordPolicy);
        app.directive('passwordPrompt', passwordPrompt);
        app.directive('share', share);
        app.directive('vouchercode', vouchercode);

        app.controller('advertDetailController', advertDetailController);
        app.controller('advertListController', advertListController);
        app.controller('categoryListController', categoryListController);
        app.controller('locationDetailController', locationDetailController);
        app.controller('locationListController', locationListController);
        app.controller('locationListGroupedByMerchantController', locationListGroupedByMerchantController);
        app.controller('merchantListController', merchantListController);
        app.controller('faqsController', faqsController);
        app.controller('termsController', termsController);
        app.controller('disclaimersController', disclaimersController);
        app.controller('instructionsController', instructionsController);
        app.controller('fencingController', fencingController);
        app.controller('headerTitleController', headerTitleController);
        app.controller('iframeController', iframeController);
        app.controller('leftMenuController', leftMenuController);
        app.controller('rightMenuController', rightMenuController);
        app.controller('settingsController', settingsController);
        app.controller('paymentDetailController', paymentDetailController);
        app.controller('paymentMethodAddController', paymentMethodAddController);
        app.controller('paymentTokenController', paymentTokenController);
        app.controller('changePasswordController', changePasswordController);
        app.controller('forgotPasswordController', forgotPasswordController);
        app.controller('activityController', activityController);
        app.controller('addressEditController', addressEditController);
        app.controller('loginController', loginController);
        app.controller('logoutController', logoutController);
        app.controller('profileController', profileController);
        app.controller('resetPasswordController', resetPasswordController);
        app.controller('userActivateController', userActivateController);
        app.controller('userEditController', userEditController);
        app.controller('userUpdateUsernameController', userUpdateUsernameController);
        app.controller('vouchercodeListByUserController', vouchercodeListByUserController);
        app.controller('vouchercodeDetailController', vouchercodeDetailController);
        app.controller('indexController', indexController);

        if (config_constants.ordering) {
            app.config(orderRoutes);
            app.config(orderStepsRoutes);
            app.constant('pre_total_gross', pre_total_gross);
            app.constant('pre_total_net', pre_total_net);
            app.constant('order_action_reason_options', order_action_reason_options);
            app.constant('order_approval_status_options', order_approval_status_options);
            app.service('menuService', menuService);
            app.service('orderingUtilService', orderingUtilService);
            app.service('orderService', orderService);
            app.service('orderResolver', orderResolver);
            app.controller('cartController', cartController);
            app.controller('discountsController', discountsController);
            app.controller('itemDetailController', itemDetailController);
            app.controller('menuListController', menuListController);
            app.controller('orderCheckoutController', orderCheckoutController);
            app.controller('orderDetailController', orderDetailController);
            app.controller('orderLocationListController', orderLocationListController);
            app.controller('orderPromocodeController', orderPromocodeController);
        }

        if (config_constants.loyalty) {
            app.config(loyaltyRoutes);
            app.service('loyaltyService', loyaltyService);
            app.service('loyaltyResolver', loyaltyResolver);
            app.controller('referFriendController', referFriendController);
            app.controller('rewardDetailController', rewardDetailController);
        }

        app.config(angular);
        app.config(angularAgility);
        app.config(auth);
        app.config(dataLayer);
        app.config(facebook);
        app.config(framework);
        app.config(google);
        app.config(http);
        app.config(moment);

        app.run(globalMethods);
        app.run(onRun);
        app.run(userEvents);
    }
);
