define([], function () {
    var http = function ($httpProvider, config_constants) {
        // It delays calling $apply on http responses and chunks multiples into one digest cycle
        $httpProvider.useApplyAsync(true);

        $httpProvider.interceptors.unshift(['$q', '$rootScope', '$window', '$injector',
            function ($q, $rootScope, $window, $injector) {
                var showingConnectionLost = false;
                var showConnectionLost = function () {
                    if (showingConnectionLost) return;
                    showingConnectionLost = true;
                    var frameworkService = $injector.get('frameworkService');
                    frameworkService.showAlert('Unable to connect to Internet. Please check your settings and retry.')
                        .then(function () {
                            showingConnectionLost = false;
                        });
                };

                var showingInvalidAppVersion = false;
                var showInvalidVersion = function () {
                    if (showingInvalidAppVersion) {
                        return;
                    }
                    showingInvalidAppVersion = true;
                    var frameworkService = $injector.get('frameworkService');
                    frameworkService.showAlert('Your version of the app needs to be updated.')
                        .then(function () {
                            if (frameworkService.getPlatform() == 'android') {
                                $window.open('market://details?id=' + config_constants.marketId.android, '_system');
                            } else if (frameworkService.getPlatform() == 'ios') {
                                $window.open('itms-apps://itunes.apple.com/app/id' + config_constants.marketId.ios, '_system');
                            }
                            showingInvalidAppVersion = false;
                        });
                };

                return {
                    request: function (config) {
                        var frameworkService = $injector.get('frameworkService');
                        frameworkService.showLoading();
                        if (_.includes(config.url, config_constants.baseAPIUrl)) {
                            config.headers = config.headers || {};
                            config.headers['x-version'] = config_constants.version;
                            if ($rootScope.geolocationEnabled) {
                                config.headers['x-device-latitude'] = _.get($rootScope, 'geolocation.latitude');
                                config.headers['x-device-longitude'] = _.get($rootScope, 'geolocation.longitude');
                            }
                        } else if (config.url === 'https://freegeoip.net/json/') {
                            config.headers = [];
                        }
                        return config;
                    },
                    response: function (response) {
                        var frameworkService = $injector.get('frameworkService');
                        frameworkService.hideLoading();
                        return _.includes(response.config.url, config_constants.baseAPIUrl) ? response.data : response;
                    },
                    responseError: function (response) {
                        var frameworkService = $injector.get('frameworkService');
                        frameworkService.hideLoading();
                        if (response.status === 401 && !_.includes(response.config.url, '/user/logout')) {
                            var userService = $injector.get('userService');
                            userService.logout().then(function () {
                                frameworkService.navigateTo('landing');
                            });
                        } else if (response.status === 404) {
                            frameworkService.showAlert('Sorry, the request failed to reach the server. Please try restarting or updating your app.');
                        } else if (response.status === 412) {
                            showInvalidVersion();
                        } else if (response.status === 500) {
                            frameworkService.showError(response);
                        } else if (response.status === 502 || response.status === -1) {
                            showConnectionLost();
                        }
                        return $q.reject(response);
                    }
                };
            }]);
    };

    http.$inject = ['$httpProvider', 'config_constants'];
    return http;
});
