define([], function () {
    var advertDetailController = function ($scope, $timeout, $location, advertResponse, frameworkService, favouriteService, keenService, localStorageService, vouchercodeService, utilityService, i18nFilter, response_enum) {
        if (_.isEmpty(advertResponse.data)) {
            frameworkService.navigateTo('advertList');
            frameworkService.showAlert('This advert is not available.');

            return;
        }

        frameworkService.showBackButton($scope);

        keenService.recordEvent('advert_views', advertResponse.data);
        $scope.advert = advertResponse.data.advert;
        $scope.advert.instance_id = advertResponse.data._id;
        $scope.advert.location = advertResponse.data.location;
        $scope.location = $scope.advert.location;
        $scope.merchant = $scope.advert.merchant;
        $scope.shareSubject = utilityService.stringFormat(i18nFilter('Share advert subject'), $scope.advert.title);
        $scope.shareMessage = utilityService.stringFormat(i18nFilter('Share advert message'), $scope.advert.title, $scope.advert.summary);

        $scope.buyNow = function () {
            keenService.recordEvent('buy_now_clicks', $scope.advert);
            if (_.includes($scope.advert.buy_now_url, 'http')) {
                frameworkService.openInBrowser($scope.advert.buy_now_url); // external URL              
            } else {
                localStorageService.store('advertInstanceId', $scope.advert.instance_id);
                $location.path($scope.advert.buy_now_url); // internal URL
            }
        };

        $scope.issueVoucher = function () {
            vouchercodeService.issueVouchercode($scope.advert.instance_id).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    // Add event for vouchers_issue
                    keenService.recordEvent('vouchers_issue', response.data);
                    frameworkService.showToast('Added to \'My coupons\'');

                    // After issuing a voucher the server adds some favourites automatically to the app user
                    favouriteService.updateFavouritesFromServer().then(function () {
                        $timeout(function () { // Wait until the first toast is hidden
                            frameworkService.showToast('Favourites updated');
                        }, 2000);
                    });

                    frameworkService.navigateTo('vouchercodeDetail', {
                        _id: response.data._id
                    });
                } else {
                    if (_.some(response.data, { message: response_enum.ERROR_CODES.VOUCHER_ISSUED_MAX_PER_USER })) {
                        frameworkService.showAlert('Sorry, you have already issued the max number of coupons for this advert.<br>Please check \'My coupons\' in your account to view your available coupons.');
                    } else if (_.some(response.data, { message: response_enum.ERROR_CODES.VOUCHERCODE_NOT_FOUND })) {
                        frameworkService.showAlert('Sorry, there are no coupons available for this advert.<br>Please try again later.');
                    } else {
                        frameworkService.showError(response.data);
                    }
                }
            });
        };
    };

    advertDetailController.$inject = ['$scope', '$timeout', '$location', 'advertResponse', 'frameworkService', 'favouriteService', 'keenService', 'localStorageService', 'vouchercodeService', 'utilityService', 'i18nFilter', 'response_enum'];
    return advertDetailController;
});
