define([], function () {
    var paymentController = function ($scope, giftcardService, giftcardUtilService, frameworkService, response_enum, userService, $window) {
        $scope.giftcards = giftcardUtilService.getGiftcards();
        if (_.isEmpty($scope.giftcards.merchant)) {
            return frameworkService.navigateTo('giftcardWelcome');
        }

        var platform;
        frameworkService.getPlatform() === 'web' ? platform = 'Web' : platform = 'App';
        giftcardUtilService.clearPreTotalNet($scope.giftcards);
        $scope.getTotalCart = giftcardUtilService.getTotalCart;
        $scope.getTotalNet = giftcardUtilService.getTotalNet;

        $scope.showPaymentMethodForm = true;
        $scope.card = {};
        if (userService.isAuthenticated()) {
            userService.getLoggedInUser().then(function (response) {
                if (!_.isNil(response.data.payment)) {
                    $scope.showPaymentMethodForm = false;
                    $scope.savedCard = response.data.payment;
                    $scope.card = angular.copy($scope.savedCard);
                }
            });
        }

        $scope.togglePaymentMethodForm = function () {
            $scope.showPaymentMethodForm = !$scope.showPaymentMethodForm;
            if (!$scope.showPaymentMethodForm) {
                $scope.card = $scope.savedCard;
            } else {
                $scope.card = {};
            }
        };

        $scope.submit = function () {
            var data = {
                card: $scope.card,
                total_cart: giftcardUtilService.getTotalCart(),
                total_net: giftcardUtilService.getTotalNet(),
                merchant: $scope.giftcards.merchant,
                customer: $scope.giftcards.customer,
                recipients: $scope.giftcards.recipients,
                pre_total_net: _.map($scope.giftcards.pre_total_net, function (m) {
                    return { type: m.type, amount: m.amount || m.amountFn(), id: m.id, name: m.name };
                }),
                upsell: _.defaultTo(_.get(_.head($scope.giftcards.pre_total_net), 'upsell'), false),
                platform: platform
            };
            var encodedData = btoa(JSON.stringify(data), 'base64');

            var windowTarget;
            var urlStatus;
            var urlReference;
            giftcardService.purchase({ data: encodedData }).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    frameworkService.showLoading();
                    frameworkService.disableNextAnimation();

                    !$window.cordova ? windowTarget = '_self' : windowTarget = '_blank';

                    var inAppBrowser = $window.open(response.data.redirect_url, windowTarget, 'location=no, toolbar=no, hidenavigationbuttons=yes');
                    if ($window.cordova) {
                        inAppBrowser.addEventListener('loadstart', loadstartCallback);
                        inAppBrowser.addEventListener('loaderror', loaderrorCallback);
                        inAppBrowser.addEventListener('exit', exitCallback);

                        function loadstartCallback(event) {
                            if (event.url.includes('success')) urlStatus = 'success';
                            if (event.url.includes('error')) urlStatus = 'error';

                            urlReference = event.url.substring(event.url.length - 15)
                            if (urlStatus) {
                                inAppBrowser.close();
                                frameworkService.navigateTo('giftcardConfirmation', { status: urlStatus, reference_id: urlReference });
                            }
                        };

                        function loaderrorCallback() {
                            inAppBrowser.close();
                            frameworkService.showAlert('Sorry, unable to load, please try again.');
                        };

                        function exitCallback() {
                            frameworkService.hideLoading();
                            inAppBrowser.removeEventListener('loadstart', loadstartCallback);
                            inAppBrowser.removeEventListener('exit', exitCallback);
                        };
                    }
                } else {
                    if (_.some(response.data, { message: response_enum.ERROR_CODES.ONESCAN_ERROR })) {
                        frameworkService.showAlert('Sorry, your purchase could not be completed at this time.');
                    } else if (_.some(response.data, { message: response_enum.ERROR_CODES.ONESCAN_SAVE_TOKEN_ERROR })) {
                        frameworkService.showAlert('Sorry, you are unable to save your card at this time. Please submit payment without saving card.');
                    } else {
                        frameworkService.showError(response.data);
                    }
                }
            });
        };
    };

    paymentController.$inject = ['$scope', 'giftcardService', 'giftcardUtilService', 'frameworkService', 'response_enum', 'userService', '$window'];
    return paymentController;
});
