define([], function () {
    var giftcardPromocodeController = function ($rootScope, $scope, moment, giftcardService, giftcardUtilService, frameworkService, response_enum) {

        var isValidForUser = function (promocode) {
            if (!promocode.is_for_full_users)
                return true;
            else if (!$rootScope.isLoggedIn)
                return false;
            else if (!promocode.limit.max_per_user)
                return true;
            else if (promocode.limit.max_per_user > promocode.used_by_user_count)
                return true;
            return false;
        };

        var isValidForGiftCard = function (promocode) {
            var amounts = _.map($scope.giftcards.recipients, 'original_amount');
            return promocode.discount.target === 'giftcard' && _.some(amounts, function (amount) {
                return _.includes(promocode.discount.giftcards, amount);
            });
        };

        var isValid = function (promocode) {
            var now = moment();

            return promocode && promocode.status === 'active'
                && now.isSameOrAfter(promocode.start_date) && now.isSameOrBefore(promocode.end_date)
                && _.includes(promocode.channels, frameworkService.getPlatform())
                && isValidForUser(promocode)
                && isValidForGiftCard(promocode);
        };

        $scope.apply = function () {
            if (_.isEmpty($scope.promocode)) return;
            giftcardService.getPromocodecampaign($scope.promocode, $scope.giftcards.merchant).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    var promocode = response.data;
                    if (!isValid(promocode)) {
                        return frameworkService.showAlert('This promo code is invalid.');
                    }

                    giftcardUtilService.removePreTotalNetByType($scope.giftcards, 'promocode');
                    var amountFn = function () {
                        return -1 * Math.min(
                            giftcardUtilService.calculateCartDiscount(promocode.discount),
                            promocode.limit.max_discount ? promocode.limit.max_discount : Infinity
                        );
                    };
                    giftcardUtilService.addPreTotalNet($scope.giftcards, {
                        type: 'promocode',
                        amountFn: amountFn,
                        name: promocode.code,
                        id: promocode._id,
                        upsell: promocode.discount.upsell
                    });

                    _.forEach($scope.giftcards.recipients, function (recipient) {
                        recipient.amount = recipient.original_amount;
                    });
                    if (promocode.discount.upsell) {
                        _.forEach($scope.giftcards.recipients, function (recipient) {
                            if (_.includes(promocode.discount.giftcards, recipient.original_amount)) {
                                recipient.amount = recipient.original_amount + Math.abs(giftcardUtilService.calculateDiscount(promocode.discount.type, recipient.original_amount, promocode.discount.amount));
                            }
                        });
                    }
                    frameworkService.showToast('Promo code applied');
                } else {
                    var msg;
                    if (_.some(response.data, { message: response_enum.ERROR_CODES.PROMOCODE_NOT_FOUND })) {
                        frameworkService.showAlert('This promo code is invalid.');
                    } else {
                        frameworkService.showError(response.data);
                    }
                }
            });
        };
    };

    giftcardPromocodeController.$inject = ['$rootScope', '$scope', 'moment', 'giftcardService', 'giftcardUtilService', 'frameworkService', 'response_enum'];
    return giftcardPromocodeController;
});
