define([], function () {
    var kidsController = function ($scope, $window, frameworkService, moment, userService, response_enum, kids_guardian_options, config_constants) {
        $scope.kids = [];

        $scope.guardians = kids_guardian_options;

        userService.getKids().then(function (response) {
            if (response.response_code == response_enum.CODE.SUCCESS) {
                $scope.kids = response.data;
                if (!_.isEmpty($scope.kids)) {
                    _.forEach($scope.kids, function (k) {
                        k.birthday = moment(k.birthday).format('DD/MM/YYYY');
                        k.tc_checked = true;
                        k.saved = true;
                    });
                } else {
                    $scope.addKid();
                }
            } else {
                frameworkService.showError(response.data);
            }
        });

        $scope.addKid = function () {
            var kid = {
                first_name: null,
                last_name: null,
                gender: 'F',
                birthday: null,
                tc_checked: false,
                saved: false
            };
            $scope.kids.push(kid);
        };

        $scope.removeKid = function (index) {
            $scope.kids.splice(index, 1);
        };

        $scope.save = function () {
            // if the user removes a child for it to be 0 and saves it it gives below alert and unable to save
            if (_.isEmpty($scope.kids)) return frameworkService.showAlert('You have not added any children.');
            if (_.some($scope.kids, { tc_checked: false })) return frameworkService.showAlert('You can only add children if you are their legal guardian.');
            if (_.some($scope.kids, { birthday: null })) return frameworkService.showAlert('You must enter your child\'s birthday.');


            var data = angular.copy($scope.kids);
            _.forEach(data, function (k) {
                k.birthday = moment.utc(k.birthday, 'DD/MM/YYYY').toISOString();
            });

            userService.saveKids(data).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {

                    // trigger appsflyer event
                    if ($window.plugins && $window.plugins.appsFlyer) {
                        $window.plugins.appsFlyer.trackEvent('st_added', {});
                    }

                    frameworkService.showToast('Kids saved');
                    frameworkService.navigateTo('profile');
                } else {
                    if (_.some(response.data, { message: response_enum.ERROR_CODES.CLIQUE_INVALID_AGE_RESTRICTION })) {
                        frameworkService.showAlert('This child does not fall inside the valid age range.');
                    } else {
                        frameworkService.showError(response);
                    }
                }
            });
        };
    };

    kidsController.$inject = ['$scope', '$window', 'frameworkService', 'moment', 'userService', 'response_enum', 'kids_guardian_options', 'config_constants'];
    return kidsController;
});
