define(['overlappingmarkerspiderfier'], function () {
    var locationListController = function ($scope, $filter, $timeout, locationsResponse, NgMap) {
        $scope.locations = _.map(locationsResponse.data, function (location) {
            // Extend locations with the full name of state and country so that it is filterable
            return _.merge(location, {
                address: {
                    state_long: $filter('bindName')(location.address.state, 'state_options'),
                    country_long: $filter('bindName')(location.address.country, 'country_options'),
                }
            });
        });

        $scope.viewMode = 'list';
        var locationMap, bounds;

        NgMap.getMap().then(function (map) {
            locationMap = map;
            locationMap.setOptions({
                styles: [{
                    featureType: 'poi.business',
                    stylers: [{ visibility: 'off' }]
                }, {
                    featureType: 'transit',
                    stylers: [{ visibility: 'off' }]
                }, {
                    elementType: 'labels.icon',
                    stylers: [{ visibility: 'off' }]
                }]
            });
            var dynamicMarkers = [];
            bounds = new google.maps.LatLngBounds();
            var oms = new OverlappingMarkerSpiderfier(map, {
                markersWontMove: true,
                markersWontHide: true,
                basicFormatEvents: true
            });
            _.forEach(locationMap.markers, function (marker) {
                bounds.extend(marker.getPosition());
                google.maps.event.addListener(marker, 'spider_click', function (e) {  // 'spider_click', not plain 'click'
                    $scope.show(e, marker.location);
                });
                oms.addMarker(marker);
            });
            locationMap.setCenter(bounds.getCenter());
            locationMap.fitBounds(bounds);
        });

        $scope.toggleViewMode = function () {
            $scope.viewMode = ($scope.viewMode === 'list') ? 'map' : 'list';
            if ($scope.viewMode === 'map' && locationMap && bounds) {
                $timeout(function () {
                    google.maps.event.trigger(locationMap, 'resize');
                    locationMap.setCenter(bounds.getCenter());
                    locationMap.fitBounds(bounds);
                });
            }
        };

        $scope.show = function (event, location) {
            $scope.selectedLocation = location;
            locationMap.showInfoWindow('map-info-window', $scope.selectedLocation._id);
        };
    };

    locationListController.$inject = ['$scope', '$filter', '$timeout', 'locationsResponse', 'NgMap'];
    return locationListController;
});
