define([], function () {
    var cartController = function ($scope, moment, frameworkService, orderingUtilService, i18nFilter) {
        $scope.order = orderingUtilService.getOrder();

        $scope.getTotalCart = orderingUtilService.getTotalCart;

        var itemBeforeEditing;
        $scope.openItemModal = function (item) {
            $scope.editMode = true;
            $scope.item = item;
            itemBeforeEditing = angular.copy(item);
            frameworkService.showModal($scope, 'views/order/itemDetailModal.html');
        };

        $scope.remove = function () {
            _.remove($scope.order.cart, $scope.item);
            orderingUtilService.setOrder($scope.order);
            frameworkService.showToast('Item removed');
            itemBeforeEditing = null; // Not needed anymore
        };
        $scope.save = function () {
            orderingUtilService.setOrder($scope.order);
            frameworkService.showToast('Item edited');
            itemBeforeEditing = null; // Not needed anymore
        };

        $scope.getStashedOrders = orderingUtilService.getStashedOrders;

        $scope.showPrompt = function () {
            frameworkService.showPopup({
                templateUrl: 'directives/templates/orderNamePrompt.html',
                title: i18nFilter('Name of order'),
                scope: $scope,
                buttons: [{
                    text: i18nFilter('Cancel'),
                    type: 'button-assertive'
                }, {
                    text: i18nFilter('OK'),
                    type: 'button-balanced',
                    onTap: function () {
                        if (_.isEmpty($scope.order.name)) {
                            return;
                        }
                        $scope.order.time = moment();
                        $scope.order.total_cart = orderingUtilService.getTotalCart();
                        orderingUtilService.stashOrder(angular.copy($scope.order));
                        orderingUtilService.clearOrder();
                        frameworkService.showToast('Order saved');
                        frameworkService.navigateToNewOrder('catering');
                    }
                }]
            });
        };

        var retrieveOrder = function (data) {
            orderingUtilService.setOrder(data);
            orderingUtilService.removeStashOrder(data);
            $scope.order = data;
            frameworkService.navigateTo('menuList', {
                ids: data.menuIds,
                menu_type: 'catering'
            }, { reload: true, inherit: false, notify: true });
        };

        $scope.retrieveOrder = function (order) {
            if (orderingUtilService.getTotalCart() > 0) {
                frameworkService.showConfirm('Retrieve order', 'The current order will be lost.', function () {
                    retrieveOrder(order);
                });
            } else {
                retrieveOrder(order);
            }
        };

        $scope.clearStashOrders = function () {
            frameworkService.showConfirm('Remove orders', 'All saved orders will be removed.', function () {
                orderingUtilService.clearStashOrders();
            })
        };

        $scope.openOrdersModal = function () {
            frameworkService.showModal($scope, 'views/order/stashedOrdersModal.html');
        };

        // Execute action on hide modal
        $scope.cancel = function () {
            if (itemBeforeEditing) {
                angular.copy(itemBeforeEditing, $scope.item); // Restore the item to the version before editing
                itemBeforeEditing = null; // Not needed anymore
            }
        };
    };

    cartController.$inject = ['$scope', 'moment', 'frameworkService', 'orderingUtilService', 'i18nFilter'];
    return cartController;
});
