define([], function () {
    var discountsController = function ($scope, frameworkService, orderingUtilService, response_enum, config_constants, giftcardService, i18nFilter, giftcardService, $element) {
        $scope.giftcards = [];

        if (config_constants.loyalty.giftcard) {
            giftcardService.getUserGiftcards().then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    var giftcardIds = _.map(response.data, 'clique_id');
                    giftcardService.getUserGiftcardsBalance(giftcardIds).then(function (response) {
                        if (response.response_code === response_enum.CODE.SUCCESS) {
                            _.remove(response.data, function (giftcard) {
                                giftcard.last4 = giftcard.clique_id.substr(giftcard.clique_id.length - 5);
                                giftcard.balance = _.find(response.data, { clique_id: giftcard.clique_id }).giftcard_balance;
                                return giftcard.balance === 0;
                            });
                        } else {
                            frameworkService.showError(response.data);
                        }
                        $scope.giftcards = response.data;
                    });
                } else {
                    frameworkService.showAlert('Unable to retrieve your linked eGift Vouchers at this time. Please enter the number manually.');
                }
            });

            $scope.onGiftcardChecked = function () {
                if (orderingUtilService.findPreTotalNetByType($scope.order, 'clique_giftcard')) this.clearGiftcards();
                this.applyGiftcard();
            };

            $scope.clearGiftcards = function () {
                $scope.giftcard.number = null;
                orderingUtilService.removePreTotalNetByType($scope.order, 'clique_giftcard');
            };

            $scope.applyGiftcard = function () {
                if (!$scope.giftcard.number) return;

                $scope.discount = { amount: null };

                checkGiftcard().then(function (boolean) {
                    if (!boolean) {
                        frameworkService.showAlert('Invalid code provided.');
                        return;
                    } else {
                        frameworkService.showPopup({
                            templateUrl: 'directives/templates/giftcardAmountPopup.html',
                            title: i18nFilter('Enter how much of your eGift Voucher you would like to use'),
                            scope: $scope,
                            buttons: [{
                                text: i18nFilter('Cancel'),
                                type: 'button-assertive',
                                onTap: function () {
                                    $scope.giftcard.number = null;
                                }
                            }, {
                                text: i18nFilter('Submit'),
                                type: 'button-balanced',
                                onTap: function ($event) {
                                    if (!$scope.discount.amount) {
                                        if ($event) $event.preventDefault();
                                        $scope.giftcard.number = null;
                                        return;
                                    }

                                    if (($scope.discount.amount * 100) > $scope.appliedGiftCardBalance) {
                                        $scope.discount.amount = $scope.appliedGiftCardBalance / 100;
                                    }

                                    orderingUtilService.removePreTotalNetByType($scope.order, 'clique_giftcard');
                                    orderingUtilService.addPreTotalNet($scope.order, {
                                        type: 'clique_giftcard',
                                        code: $scope.giftcard.number,
                                        amountFn: $scope.usableGiftcard
                                    });
                                    frameworkService.showToast('Giftcard amount removed from cart');
                                }
                            }]
                        });
                    }
                });
            };

            var checkGiftcard = function () {
                return giftcardService.getUserGiftcardsBalance([$scope.giftcard.number]).then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        if (_.head(response.data).giftcard_balance > 0) {
                            $scope.appliedGiftCardBalance = _.head(response.data).giftcard_balance;
                            return true;
                        }

                        return false;
                    } else {
                        frameworkService.showError(response.data);
                    }
                });
            };

            $scope.usableGiftcard = function () {
                return -1 * Math.min(
                    $scope.discount.amount * 100,
                    orderingUtilService.getTotalGrossPlusSomeTypes($scope.order, ['promocode', 'clique_voucher'])
                );
            };
        }
    };

    discountsController.$inject = ['$scope', 'frameworkService', 'orderingUtilService', 'response_enum', 'config_constants', 'giftcardService', 'i18nFilter', 'giftcardService', '$element'];
    return discountsController;
});
