define([], function () {
    var menuListController = function ($scope, $filter, $stateParams, frameworkService, orderingUtilService, favouriteService, menuResponse, i18nFilter, config_constants, $timeout) {
        $scope.itemsOnly = false;

        if (_.isEmpty(menuResponse.data)) {
            frameworkService.navigateToNewOrder();
            frameworkService.showAlert('This menu has been modified. Please try again.');
            return;
        }
        $scope.order = orderingUtilService.getOrder();

        var menus;
        if (config_constants.ordering.flow === 'menus_first') {
            menus = menuResponse.data;
        } else {
            menus = menuResponse.data.menus;
            $scope.location = _.get($scope, 'order.location');
            $scope.merchant = _.get($scope, 'order.location.merchant');
        }

        var favouriteItems = [];
        var favouritesSKU = _.map(_.filter(favouriteService.getFavourites(), { type: 'item' }), 'document_id');
        var urlItem;
        _.forEach(menus, function (menu) {
            _.forEach(menu.sections, function (section) {
                _.remove(section.items, function (item) { // the section might refer to inactive or deleted items
                    var fullItem = _.find(menu.items, { _id: item._id });
                    if (fullItem) {
                        _.merge(item, fullItem);
                        item.menu = menu._id; // menuInstanceId
                        item.menu_id = menu.menu_id; // menuId
                        item.section = section._id;
                        if (item._id === $stateParams.item_id) {
                            urlItem = item;
                        }
                        var favouriteVariant = _.find(item.variant_group.variants, function (variant) {
                            return _.includes(favouritesSKU, variant.sku);
                        });
                        if (favouriteVariant) {
                            favouriteItems.push(item);
                            // An SKU can be used in more than one section, so we need to make sure the favouritesItems is a set
                            _.pull(favouritesSKU, favouriteVariant.sku);
                        }
                    }
                    return !fullItem;
                });

                _.forEach(section.subsections, function (subsection) {
                    _.remove(subsection.items, function (item) { // the section might refer to inactive or deleted items
                        var fullItem = _.find(menu.items, { _id: item._id });
                        if (fullItem) {
                            _.merge(item, fullItem);
                            item.menu = menu._id; // menuInstanceId
                            item.menu_id = menu.menu_id; // menuId
                            item.section = subsection._id;
                            if (item._id === $stateParams.item_id) {
                                urlItem = item;
                            }
                            var favouriteVariant = _.find(item.variant_group.variants, function (variant) {
                                return _.includes(favouritesSKU, variant.sku);
                            });
                            if (favouriteVariant) {
                                favouriteItems.push(item);
                                // An SKU can be used in more than one section, so we need to make sure the favouritesItems is a set
                                _.pull(favouritesSKU, favouriteVariant.sku);
                            }
                        }
                        return !fullItem;
                    });
                });
            });
        });
        $scope.menus = menus;
        $scope.favouriteItems = favouriteItems;
        $scope.filter = {};

        $scope.isGroupShown = function (key) {
            return $scope.filter.shownGroup === key;
        };

        $scope.hasSubSections = function (section) {
            return _.size(section.subsections) > 0
        };

        $scope.isSubGroupShown = function (key) {
            return $scope.filter.shownSubGroup === key;
        };

        $scope.toggleSubGroup = function (key) {
            $scope.filter.shownSubGroup = ($scope.isSubGroupShown(key) && frameworkService.getPlatform() !== 'web') ? null : key;
            frameworkService.resizeScrollableWindow();
        };

        $scope.toggleGroup = function (key) {
            if (key !== 'favourites') frameworkService.navigateTo('menuSection', { section_id: key }, { notify: false });
            $scope.filter.shownGroup = ($scope.isGroupShown(key) && frameworkService.getPlatform() !== 'web') ? null : key;
            $scope.shownGroupImage = _.get(_.find($scope.menus, { _id: $scope.filter.shownGroup }), 'hero_image.path_to_file');
            frameworkService.resizeScrollableWindow();
        };
        if (frameworkService.getPlatform() === 'web' || config_constants.ordering.readonly) $scope.toggleGroup(_.get(_.head($scope.menus), '_id'));

        $scope.showItems = function (section) {
            $scope.selectedSection = section;
            $scope.itemsOnly = true;
            frameworkService.scrollTop();
        };

        $scope.resetMenu = function (event) {
            if (event) {
                event.preventDefault();  // needed as checkout button click was firing due to event bubbling
                event.stopImmediatePropagation();
                $timeout(resetValues, 300);  // needed due to $scope inside setTimeout not updating
            } else {
                resetValues();    // read only menus
            }
        };

        var resetValues = function () {
            $scope.selectedSection = null;
            $scope.itemsOnly = false;
            frameworkService.scrollTop();
        };

        $scope.getTotalCart = orderingUtilService.getTotalCart;

        $scope.openItemModal = function (item) {
            if (config_constants.ordering.readonly) return;
            frameworkService.navigateTo('menuItem', { item_id: item._id }, { notify: false });
            $scope.item = angular.copy(item);
            $scope.item.selected = {
                variant: $scope.item.variant_group.variants[0],
                addon_groups: [],
                modifiers: _.map($scope.item.modifiers, function (m) {
                    return {
                        name: m.name,
                        value: m.values[0]
                    };
                }),
                quantity: 1
            };
            frameworkService.showModal($scope, 'views/order/itemDetailModal.html');
        };

        var menu_type = _.defaultTo($stateParams.menu_type, 'standard');
        function resetCart() {
            $scope.order.cart = [];
            $scope.order.type = menu_type;
        }

        function addToCart() {
            $scope.order.cart.push($scope.item);
            orderingUtilService.setOrder($scope.order);
            frameworkService.showToast('Added to cart');
            frameworkService.navigateTo('^', {}, { notify: false }); // Go to parent
        };
        $scope.add = function () {
            if (!$scope.order.cart) {
                resetCart();
            }
            // reset cart if type changes
            if ($scope.order.type !== menu_type) {
                frameworkService.showConfirm('Confirm', 'All items in cart will be deleted because the order type changed.', function () {
                    resetCart();
                    addToCart();
                });
            } else {
                addToCart();
            }
        };
        // Execute action on hide modal
        $scope.cancel = function () {
            frameworkService.navigateTo('^', {}, { notify: false }); // Go to parent
        };

        $scope.checkout = function () {
            if (config_constants.ordering.flow === 'menus_first') {
                frameworkService.navigateTo('orderLocationList', { menu_type: menu_type });
            } else {
                frameworkService.navigateTo('orderCheckout');
            }
        };

        if (frameworkService.getPlatform() === 'web') { // In mobile, don't open the first section of the menu
            if (!_.isEmpty($stateParams.section_id)) {
                $scope.toggleGroup($stateParams.section_id);
            } else if (!_.isEmpty($scope.favouriteItems)) {
                $scope.toggleGroup('favourites'); // Show the favourites
            } else {
                var firstMenu = _.minBy($scope.menus, 'sort_number');
                var firstSection = _.minBy(firstMenu.sections, 'sort_number');
                $scope.toggleGroup(firstSection._id); // Show the first section of the first menu

                if (firstSection.subsections.length > 0) $scope.toggleSubGroup(firstSection.subsections[0]._id);
            }
        }

        if (urlItem) {
            $scope.openItemModal(urlItem);
        }
    };

    menuListController.$inject = ['$scope', '$filter', '$stateParams', 'frameworkService', 'orderingUtilService', 'favouriteService', 'menuResponse', 'i18nFilter', 'config_constants', '$timeout'];
    return menuListController;
});
