define([], function () {
    var orderCheckoutController = function ($rootScope, $scope, $filter, orderingUtilService, orderService, paymentService, addressService, userService, favouriteService, frameworkService, keenService, addressesResponse, response_enum, i18nFilter, country_options, state_options, config_constants, $window, utilityService, giftcardService) {
        $scope.order = orderingUtilService.getOrder();
        if (_.isEmpty($scope.order.cart)) {
            return frameworkService.navigateTo(_.get(config_constants, 'ordering.flow') === 'menus_first' ? 'menuList' : 'orderLocationList');
        }

        var platform;
        frameworkService.getPlatform() === 'web' ? platform = 'Web' : platform = 'App';

        $scope.showPaymentMethodForm = false;
        $scope.card = {};
        if (userService.isAuthenticated()) {
            userService.getLoggedInUser().then(function (response) {
                if (!_.isNil(response.data.payment)) {
                    $scope.showPaymentMethodForm = false;
                    $scope.savedCard = response.data.payment;
                    $scope.card = angular.copy($scope.savedCard);
                } else {
                    $scope.showPaymentMethodForm = true;
                }
            });
        }

        orderingUtilService.clearPreTotalNet($scope.order);
        $scope.location = _.get($scope.order, 'location');
        $scope.merchant = _.get($scope.order, 'location.merchant');

        $scope.remember = {
            card: $rootScope.isLoggedIn
        };

        $scope.giftcard = {
            number: null
        };

        $scope.togglePaymentMethodForm = function () {
            $scope.showPaymentMethodForm = !$scope.showPaymentMethodForm;
            if (!$scope.showPaymentMethodForm) {
                $scope.card = $scope.savedCard;
            } else {
                $scope.card = {};
            }
        };

        if ($scope.order.collect.method === 'delivery') {
            $scope.deliveryAddresses = _.get(addressesResponse, 'data.delivery');
            if (!_.isEmpty($scope.deliveryAddresses)) {
                $scope.order.delivery_address = _.head($scope.deliveryAddresses);
            } else {
                $scope.order.catering_address = $scope.address;
            }

            $scope.countries = country_options;
            $scope.states = state_options;

            $scope.showDeliveryAddressForm = _.isEmpty($scope.deliveryAddresses);
            $scope.remember.deliveryAddress = $rootScope.isLoggedIn;

            $scope.toggleDeliveryAddressForm = function () {
                $scope.showDeliveryAddressForm = !$scope.showDeliveryAddressForm;
            };
        }

        $scope.customer = $rootScope.isLoggedIn ? null : _.extend({}, $scope.order.customer);
        $scope.order.customer = $scope.customer; // Backup the customer for catering (restore order)

        $scope.getTotalCart = orderingUtilService.getTotalCart;
        $scope.getTotalGross = orderingUtilService.getTotalGross;
        $scope.getTotalNet = orderingUtilService.getTotalNet;

        $scope.submit = function () {
            if ($scope.getTotalCart() < $scope.location.ordering[$scope.order.collect.method].min_amount) {
                frameworkService.showAlert(
                    i18nFilter('The minimum for ordering in this location is ') + $filter('enhancedCurrency')($scope.location.ordering[$scope.order.collect.method].min_amount) + '. ' +
                    i18nFilter('Please add enough items to reach that amount and try again.')
                );
                return;
            } else if (!$rootScope.isLoggedIn && !$scope.customer.tc_checked) {
                frameworkService.showAlert('You need to accept the terms and conditions.');
                return;
            }

            var data = {
                platform: platform,
                payment: {
                    amount: $scope.getTotalCart(),
                    provider: 'paygate'
                },
                card: $scope.card,
                customer: $scope.customer,
                total_cart: $scope.getTotalCart(),
                pre_total_gross: _.map($scope.order.pre_total_gross, function (m) {
                    return { type: m.type, amount: m.amount || m.amountFn(), id: m.id, name: m.name };
                }),
                total_gross: $scope.getTotalGross(),
                pre_total_net: _.map($scope.order.pre_total_net, function (m) {
                    return { type: m.type, amount: m.amount || m.amountFn(), id: m.id, name: m.name, code: m.code };
                }),
                total_net: $scope.getTotalNet(),
                delivery_address: $scope.order.delivery_address,
                location: $scope.order.location._id,
                merchant: $scope.order.location.merchant._id,
                collect: _.pick($scope.order.collect, ['method', 'time']),
                cart: _.map($scope.order.cart, function (item) {
                    return {
                        _id: item._id,
                        selected: {
                            sku: item.selected.variant.sku,
                            price: item.selected.variant.price,
                            quantity: item.selected.quantity,
                            notes: item.selected.notes,
                            addons: _.map(_.flatMap(item.selected.addon_groups, 'addons'), function (addon) {
                                return {
                                    sku: addon.variant.sku,
                                    price: addon.variant.price,
                                    quantity: addon.quantity,
                                    notes: _.get(addon.notes, 'value')
                                };
                            }),
                            modifiers: item.selected.modifiers
                        },
                        menu: item.menu,
                        section: item.section
                    };
                }),
                type: $scope.order.type
            };

            if ($scope.order.collect.method === 'delivery' && $scope.showDeliveryAddressForm) data.delivery_address = $scope.address;

            var submitOrder = $scope.order.type === 'catering' ? submitCateringOrder : submitStandardOrder;

            if ($scope.getTotalNet() == 0) {
                frameworkService.navigateToHome();
                return;
            }

            paymentService.authorisePayment(data).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    frameworkService.showLoading();
                    frameworkService.disableNextAnimation();

                    data.onescan_reference = utilityService.getParameterByName('PAY_REQUEST_ID', response.data.redirect_url);

                    submitOrder(data).then(function (orderResponse) {
                        if (!orderResponse.addOrderSuccess) {
                            frameworkService.hideLoading();
                            return;
                        }

                        !$window.cordova ? windowTarget = '_self' : windowTarget = '_blank';
                        var inAppBrowser = $window.open(response.data.redirect_url, windowTarget, 'location=no');
                        if ($window.cordova) {
                            inAppBrowser.addEventListener('loadstart', loadstartCallback);
                            inAppBrowser.addEventListener('loaderror', loaderrorCallback);
                            inAppBrowser.addEventListener('exit', exitCallback);

                            var urlStatus;
                            var orderId = orderResponse._id;
                            function loadstartCallback(event) {
                                if (event.url.includes('success')) urlStatus = 'success';
                                if (event.url.includes('error')) urlStatus = 'error';

                                if (urlStatus) {
                                    inAppBrowser.close();
                                    frameworkService.navigateTo('orderDetail', { _id: orderId, paymentStatus: urlStatus });
                                }
                            };

                            function loaderrorCallback() {
                                inAppBrowser.close();
                                frameworkService.showAlert('Sorry, unable to load, please try again.');
                            };

                            function exitCallback() {
                                frameworkService.hideLoading();
                                inAppBrowser.removeEventListener('loadstart', loadstartCallback);
                                inAppBrowser.removeEventListener('exit', exitCallback);
                            };
                        }
                    });
                } else {
                    if (_.some(response.data, { message: response_enum.ERROR_CODES.ONESCAN_ERROR })) {
                        frameworkService.showAlert('Sorry, your purchase could not be completed at this time.');
                    } else {
                        frameworkService.showError(response.data);
                    }
                }
            });
        };

        var submitStandardOrder = function (data) {
            var successfulOrder = false;
            return orderService.addOrder(data).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    successfulOrder = true;

                    if ($rootScope.isLoggedIn) {
                        favouriteService.updateFavouritesFromServer();
                        if ($scope.order.collect.method === 'delivery' && $scope.showDeliveryAddressForm && $scope.remember.deliveryAddress) {
                            addressService.saveAddress(data.delivery_address);
                        }
                    }

                    keenService.recordEvent('activities_add', {
                        source_type: 'order',
                        source: data,
                        total_gross: data.total_gross,
                        total_net: data.total_net,
                        merchant_id: data.merchant,
                        location_id: data.location
                    });

                    _.forEach(data.pre_total_net, function (preTotalNet) {
                        if (preTotalNet.type === 'promocode') {
                            keenService.recordEvent('promocodes_use', {
                                _id: preTotalNet.id,
                                amount: preTotalNet.amount,
                                code: preTotalNet.name,
                                merchant_id: data.merchant,
                                location_id: data.location
                            });
                        }
                    });
                } else if (_.some(response.data, { message: response_enum.ERROR_CODES.INVALID_COLLECT_TIME })) {
                    frameworkService.showAlert('Sorry, your order has timed out. Please restart your order.')
                        .then(function () {
                            orderingUtilService.clearOrder();
                            frameworkService.navigateToNewOrder();
                        });
                } else if (_.some(response.data, { message: response_enum.ERROR_CODES.LOCATION_CAN_NOT_DELIVER })) {
                    frameworkService.showAlert('Sorry, the location selected can not deliver to your address. Please select another address or choose a different location.');
                } else if (_.some(response.data, { message: response_enum.ERROR_CODES.DELIVERY_FEE_DIFFERS })) {
                    frameworkService.showAlert('Sorry, your delivery address does not match the suburb selected before. Please revise your delivery address, or return to the previous step to set a different suburb.');
                } else {
                    var msg = _.get(_.head(response.data), 'message');
                    if (!_.isEmpty(msg)) {
                        frameworkService.showAlert(msg);
                    } else {
                        frameworkService.showError(response.data);
                    }
                }
                return { _id: response.data._id, addOrderSuccess: successfulOrder };
            });
        };

        $scope.toggleGroup = function (key) {
            $scope.shownGroup = $scope.isGroupShown(key) ? null : key;
        };
        $scope.isGroupShown = function (key) {
            return $scope.shownGroup === key;
        };
    };

    orderCheckoutController.$inject = ['$rootScope', '$scope', '$filter', 'orderingUtilService', 'orderService', 'paymentService', 'addressService', 'userService', 'favouriteService', 'frameworkService', 'keenService', 'addressesResponse', 'response_enum', 'i18nFilter', 'country_options', 'state_options', 'config_constants', '$window', 'utilityService', 'giftcardService'];
    return orderCheckoutController;
});
