define([], function () {
    var orderDetailController = function ($scope, $stateParams, config_constants, orderResponse, frameworkService) {
        $scope.$on('$stateChangeSuccess', function (event, toState, toParams, fromState, fromParams) {
            $scope.showContinueButton = fromState.name === 'orderCheckout';
        });

        switch ($stateParams.paymentStatus) {
            case 'success':
                frameworkService.showToast('Payment successful');
                break;
            case 'error':
                frameworkService.showAlert('Payment unsuccessful');
                break;
        };

        var _parseOrder = function (order) {
            order.customerNotes = _.filter(order.notes, function (n) {
                return n.user._id == order.submitted_by;
            });
            order.approvalStatusReason = _.findLast(order.action_history, { action: order.approval_status }).reason;

            // Transform a {key, value} array into a dictionary
            if (!_.isEmpty(order.payment.details))
                order.payment_details = _.zipObject(_.map(order.payment.details, 'key'), _.map(order.payment.details, 'value'));

            order.loyalty_earned = _.chain(order.action_history).find({ action: 'loyalty_earn_captured' }).get('details').find({ key: 'amount' }).get('value').defaultTo(0).toNumber().value();

            _.forEach(order.cart, function (item) {
                var selectedAddonGroups = [];
                _.forEach(item.addon_groups, function (addon_group) {
                    var selectedAddons = _.filter(addon_group.addons, function (addon) {
                        return _.some(item.selected.addons, function (selected_addon) {
                            return selected_addon.sku === addon.variant_group.variants[0].sku;
                        });
                    });
                    if (!_.isEmpty(selectedAddons)) {
                        selectedAddonGroups.push({
                            name: addon_group.name,
                            addons: _.map(selectedAddons, function (selectedAddon) {
                                return {
                                    name: selectedAddon.name,
                                    variant: selectedAddon.variant_group.variants[0],
                                    quantity: 1
                                };
                            })
                        });
                    }
                });

                item.selected.variant = _.find(item.variant_group.variants, { sku: item.selected.sku });
                item.selected.addon_groups = selectedAddonGroups;
            });
        };
        _parseOrder(orderResponse.data);

        $scope.order = orderResponse.data;
        $scope.location = $scope.order.location_snapshot;
        $scope.merchant = $scope.order.merchant_snapshot;

        $scope.invoiceUrl = config_constants.baseAPIUrl + '/order/pdf/' + $scope.order._id;
    };

    orderDetailController.$inject = ['$scope', '$stateParams', 'config_constants', 'orderResponse', 'frameworkService'];
    return orderDetailController;
});
