define([], function () {
    var orderLocationListController = function ($rootScope, $scope, $stateParams, moment, orderingUtilService, orderResolver, fencingService, frameworkService, utilityService, config_constants, i18nFilter, response_enum) {
        $scope.order = orderingUtilService.getOrder();

        if (!_.get($scope, 'order.collect.method')) {
            _.set($scope, 'order.collect.method', 'pickup');
        }
        $scope.locations = [];
        var locationsFetcher = utilityService.createEmptyFetcher();
        $scope.moreDataCanBeLoaded = false;

        $scope.placeRestrictions = { country: fencingService.getCountry() };

        $scope.placeChanged = function () {
            $scope.order.collect.googlePlace = this.getPlace();
        };

        var collectWatcher = $scope.$watchGroup(['order.collect.method', 'order.collect.googlePlace'], function () {
            var geolocation = _.get($scope, 'order.collect.googlePlace.geometry.location');
            if (geolocation) {
                geolocation = {
                    latitude: _.result(geolocation, 'lat'),
                    longitude: _.result(geolocation, 'lng')
                };

            } else {
                geolocation = $rootScope.geolocation || { latitude: 0, longitude: 0 };
            }
            $scope.locations = [];
            if (_.get($scope, 'order.collect.method') === 'pickup') {
                locationsFetcher = orderResolver.getPickupMenus($stateParams.menu_type, geolocation.latitude, geolocation.longitude);
            } else {
                var postcode = _.get(_.find(_.get($scope, 'order.collect.googlePlace.address_components'), function (component) {
                    return _.includes(component.types, 'postal_code');
                }), 'long_name');
                if (!postcode) return frameworkService.showAlert('Please select a valid suburb.');
                locationsFetcher = orderResolver.getDeliveryMenus($stateParams.menu_type, geolocation.latitude, geolocation.longitude, postcode);
            }
            // The infinite scrolls (desktop and app) do not interact well with dynamic collections (no scroll event). That's why we have to call it manually
            $scope.loadMoreData();
        });

        $scope.loadMoreData = function () {
            frameworkService.loadMoreData($scope, locationsFetcher)
                .then(function (response) {
                    if (response != null) {
                        $scope.moreDataCanBeLoaded = true;
                        _.forEach(response.data, function (instance) {
                            _.forEach(instance.menus, function (menu) {
                                if ($scope.order.collect.method === 'pickup') {
                                    menu.hours = menu.pickup_hours;
                                } else if ($scope.order.collect.method === 'delivery') {
                                    menu.hours = menu.delivery_hours;
                                }
                            });
                            instance.location.menus = instance.menus;

                            var now = moment.tz(instance.location.timezone);
                            var todayOpeningHours = _.map(_.filter(instance.location.opening_hours, { day: utilityService.getDayName(now, instance.location.timezone) }), function (openingHours) {
                                return {
                                    start: utilityService.setTime(now, openingHours.start, instance.location.timezone),
                                    end: utilityService.setTime(now, openingHours.end, instance.location.timezone)
                                };
                            });
                            instance.location.isOpenNow = _.some(todayOpeningHours, function (openingHours) {
                                return now.isSameOrAfter(openingHours.start) && now.isSameOrBefore(openingHours.end);
                            });
                            $scope.locations.push(instance.location);
                        });
                    } else {
                        if ($scope.order.collect.method === 'delivery' && _.isEmpty($scope.locations)) {
                            frameworkService.showAlert('Sorry, delivery is unavailable for the suburb selected.');
                        }
                    }
                });
        };

        var dateWatcher = $scope.$watch('datepickerOptions.inputDate', function (newValue, oldValue) {
            if (!newValue)
                return;

            // Populate dropdown with the times from the intervals in steps of 15 minutes
            var selectedDay = moment(newValue);
            var locationTimezone = $scope.data.location.timezone;
            var hourIntervals = _.filter(_.flatMap($scope.data.location.menus, 'hours'), { day: utilityService.getDayName(selectedDay, locationTimezone) });
            var selectableHours = [];
            _.forEach(hourIntervals, function (interval) {
                var hour = utilityService.setTime(selectedDay, interval.start, locationTimezone);
                var endHour = utilityService.setTime(selectedDay, interval.end, locationTimezone);
                if ($scope.data.isRange) {
                    selectableHours.push({
                        start: hour,
                        end: endHour
                    });
                } else {
                    while (hour.isSameOrBefore(endHour)) {
                        selectableHours.push(angular.copy(hour));
                        hour.add(15, 'minutes');
                    }
                }
            });

            // Sort and remove duplicates
            selectableHours = _.sortedUniqBy(_.sortBy(selectableHours, function (h) {
                return $scope.data.isRange ? h.start : h;
            }), function (h) {
                return $scope.data.isRange ? h.start.format('h:mm A') : h.format('h:mm A');
            });

            $scope.dropdownOptions = _.filter(selectableHours, function (h) {
                var time = $scope.data.isRange ? h.start : h;
                return time.isSameOrAfter($scope.data.minTimeToCollect) && time.isSameOrBefore($scope.data.maxTimeToCollect);
            });

            if (_.isEmpty($scope.dropdownOptions)) {
                frameworkService.showAlert('No times available for the day selected.');
                $scope.datepickerOptions.inputDate = oldValue;
            } else {
                $scope.data.time = _.head($scope.dropdownOptions); // Select the earliest time to collect
            }
        });

        $scope.selectTime = function (location) {
            var minTimeToCollect = moment
                .tz(location.timezone)
                .add(15, 'minutes') // Add 15 minutes (avg time for the user to submit the order)
                .add(location.ordering[$scope.order.collect.method].min_minutes, 'minutes'); // Add minimum time to get the order ready for pickup or delivered

            var maxTimeToCollect = moment
                .tz(location.timezone)
                .add(location.ordering[$scope.order.collect.method].max_minutes, 'minutes');

            $scope.data = {
                time: null,
                minTimeToCollect: minTimeToCollect,
                maxTimeToCollect: maxTimeToCollect,
                isRange: location.ordering[$scope.order.collect.method].is_range,
                location: location
            };
            var selectedDay = moment.tz(location.timezone).startOf('day');
            var hasSelectableHours = false;
            while (!hasSelectableHours && selectedDay.isSameOrBefore(maxTimeToCollect, 'day')) {
                var isHoliday = _.some(location.holidays, function (holiday) {
                    return selectedDay.isSame(holiday, 'day');
                });
                if (!isHoliday) {
                    var hourIntervals = _.filter(_.flatMap(location.menus, 'hours'), { day: utilityService.getDayName(selectedDay, location.timezone) });
                    hasSelectableHours = _.some(hourIntervals, function (interval) {
                        var startHour = utilityService.setTime(selectedDay, interval.start, location.timezone);
                        var endHour = utilityService.setTime(selectedDay, interval.end, location.timezone);
                        if ($scope.data.isRange) {
                            return startHour.isSameOrAfter($scope.data.minTimeToCollect) && startHour.isSameOrBefore($scope.data.maxTimeToCollect);
                        } else {
                            return startHour.isSameOrBefore($scope.data.maxTimeToCollect) && endHour.isSameOrAfter($scope.data.minTimeToCollect);
                        }
                    });
                }
                if (!hasSelectableHours) {
                    selectedDay.add(1, 'day');
                }
            }
            if (!hasSelectableHours) {
                frameworkService.showAlert('No times available for the location selected. Please select another location.');
                return;
            }

            $scope.datepickerOptions = frameworkService.getDatepickerOptions({
                start: minTimeToCollect,
                end: maxTimeToCollect,
                date: selectedDay,
                disabled: location.holidays,
                timezone: location.timezone
            });

            frameworkService.showPopup({
                templateUrl: 'views/order/orderTimePopup.html',
                title: i18nFilter('Select order date/time'),
                scope: $scope,
                buttons: [{
                    text: i18nFilter('Cancel'),
                    type: 'button-assertive'
                }, {
                    text: i18nFilter('OK'),
                    type: 'button-balanced',
                    onTap: function (event) {
                        if ($scope.data.time) {
                            var selectedDay = $scope.datepickerOptions.inputDate;
                            if ($scope.data.isRange) {
                                $scope.order.collect.time = {
                                    start: $scope.data.time.start.toISOString(),
                                    end: $scope.data.time.end.toISOString()
                                };
                            } else {
                                $scope.order.collect.time = {
                                    start: $scope.data.time.toISOString(),
                                    end: $scope.data.time.toISOString()
                                };
                            }
                            $scope.order.location = location;
                            orderingUtilService.clearPreTotalGross($scope.order);

                            if ($scope.order.collect.method === 'delivery' && location.ordering.delivery.fee) {
                                orderingUtilService.addPreTotalGross($scope.order, {
                                    type: 'delivery_fee',
                                    amount: location.ordering.delivery.fee
                                });
                                var isWeekend = moment.tz(selectedDay, location.timezone).day() % 6 == 0;
                                if (isWeekend && location.ordering.delivery.weekend_fee_multiplier != 1) {
                                    orderingUtilService.addPreTotalGross($scope.order, {
                                        type: 'weekend_surcharge',
                                        amount: _.ceil(location.ordering.delivery.fee * (location.ordering.delivery.weekend_fee_multiplier - 1))
                                    });
                                }
                            }

                            if (config_constants.ordering.flow === 'locations_first') {
                                var menuIds = [];
                                var selectedHour = $scope.order.collect.time.start;
                                _.forEach(location.menus, function (menu) {
                                    var availableHoursIntervals = _.filter(menu.hours, { day: utilityService.getDayName(selectedDay, location.timezone) });
                                    if (_.some(availableHoursIntervals, function (interval) {
                                        var startHour = utilityService.setTime(selectedDay, interval.start, location.timezone);
                                        var endHour = utilityService.setTime(selectedDay, interval.end, location.timezone);
                                        return startHour.isSameOrBefore(selectedHour) && endHour.isSameOrAfter(selectedHour);
                                    })) {
                                        menuIds.push(menu._id);
                                    }
                                });
                                $scope.order.menuIds = menuIds; // Used for the steps tabs in the webapp
                                $scope.order.cart = [];
                                $scope.order.type = $stateParams.menu_type;
                                orderingUtilService.setOrder($scope.order);
                                frameworkService.navigateTo('menuList', {
                                    _ids: menuIds,
                                    menu_type: $stateParams.menu_type
                                });
                            } else {
                                orderingUtilService.setOrder($scope.order);
                                frameworkService.navigateTo('orderCheckout');
                            }
                        } else if (event) {
                            event.preventDefault();
                        }
                    }
                }]
            });
        };

        // Used for ordering readonly mode
        $scope.selectMenu = function (location) {
            var menuIds = [];
            _.forEach(location.menus, function (menu) {
                menuIds.push(menu._id);
            });
            $scope.order.location = location;
            $scope.order.menuIds = menuIds; // Used for the steps tabs in the webapp
            $scope.order.cart = [];
            $scope.order.type = $stateParams.menu_type;
            orderingUtilService.setOrder($scope.order);
            frameworkService.navigateTo('menuList', {
                _ids: menuIds,
                menu_type: $stateParams.menu_type
            });
        };

        // Cleanup
        $scope.$on('$destroy', function () {
            collectWatcher();
            dateWatcher();
        });
    };

    orderLocationListController.$inject = ['$rootScope', '$scope', '$stateParams', 'moment', 'orderingUtilService', 'orderResolver', 'fencingService', 'frameworkService', 'utilityService', 'config_constants', 'i18nFilter', 'response_enum'];
    return orderLocationListController;
});
