define([], function () {
    var orderPromocodeController = function ($rootScope, $scope, moment, orderService, orderingUtilService, frameworkService, response_enum) {

        var isValidForUser = function (promocode) {
            if (!promocode.is_for_full_users)
                return true;
            else if (!$rootScope.isLoggedIn)
                return false;
            else if (!promocode.limit.max_per_user)
                return true;
            else if (promocode.limit.max_per_user > promocode.used_by_user_count)
                return true;
            return false;
        };

        var isValidForOrder = function (promocode, order) {
            if (promocode.discount.target === 'delivery_fee' && _.get(order, 'collect.method') !== 'delivery') {
                return false;
            }
            return true;
        };

        var isValid = function (promocode) {
            var now = moment();
            // switching timezone of dates without changing the values. For example 2017-02-23T4:50:00Z(UTC) to 2017-02-23T4:50:00+11:00(Sydney)
            var startDate = moment.tz(promocode.start_date, 'YYYY-MM-DDTHH:mm:ss', $scope.order.location.timezone);
            var endDate = moment.tz(promocode.end_date, 'YYYY-MM-DDTHH:mm:ss', $scope.order.location.timezone);

            return promocode && promocode.status === 'active'
                && now.isSameOrAfter(startDate) && now.isSameOrBefore(endDate)
                && _.includes(promocode.channels, frameworkService.getPlatform())
                && isValidForUser(promocode)
                && isValidForOrder(promocode, $scope.order);
        };

        $scope.apply = function () {
            if (_.isEmpty($scope.promocode)) return;
            orderService.getPromocodecampaign($scope.promocode, $scope.order.location.merchant._id)
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        var promocode = response.data;
                        if (!isValid(promocode)) {
                            return frameworkService.showAlert('This promo code is invalid.');
                        }

                        var amountFn = function () {
                            return -1 * Math.min(
                                orderingUtilService.calculateDiscount(promocode.discount),
                                promocode.limit.max_discount ? promocode.limit.max_discount : Infinity
                            );
                        };
                        orderingUtilService.removePreTotalNetByType($scope.order, 'promocode');
                        orderingUtilService.addPreTotalNet($scope.order, {
                            type: 'promocode',
                            amountFn: amountFn,
                            name: promocode.code,
                            id: promocode._id
                        });
                        frameworkService.showToast('Promo code applied');
                    } else {
                        if (_.some(response.data, { message: response_enum.ERROR_CODES.PROMOCODE_NOT_FOUND })) {
                            frameworkService.showAlert('This promo code is invalid.');
                        } else {
                            frameworkService.showError(response.data);
                        }
                    }
                });
        };
    };

    orderPromocodeController.$inject = ['$rootScope', '$scope', 'moment', 'orderService', 'orderingUtilService', 'frameworkService', 'response_enum'];
    return orderPromocodeController;
});
