define([], function () {
    var paymentTokenController = function ($scope, $timeout, loyaltyService, frameworkService, paymentService, paymentMethodsResponse, i18nFilter, response_enum, utilityService) {
        $scope.defaultPaymentMethod = _.head(paymentMethodsResponse.data);

        $scope.selected = {
            card: !_.isEmpty($scope.defaultPaymentMethod),
            cash: _.isEmpty($scope.defaultPaymentMethod)
        };

        $scope.getToken = function () {
            loyaltyService.getBalance()
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        $scope.balance = response.data.available_balance;

                        if (_.isNil($scope.selected.cashback))
                            $scope.selected.cashback = $scope.balance > 0;

                        var data = {
                            cashback_amount: $scope.selected.cashback ? $scope.balance : 0
                        };
                        if ($scope.selected.cash) {
                            data.payment_method = 'in_store';
                        } else {
                            data.payment_method = 'in_app';
                            data.payment_provider = 'stripe';
                            data.stripe_token = _.get($scope.defaultPaymentMethod, 'id');
                        }
                        paymentService.generatePaymentToken(data)
                            .then(function (response) {
                                if (response.response_code === response_enum.CODE.SUCCESS) {
                                    $scope.token = response.data.token;
                                    utilityService.clearQRcode('qr');
                                    utilityService.generateQRcode('qr', $scope.token, 100, 100);
                                } else {
                                    frameworkService.showError(response.data);
                                }
                            });
                    } else {
                        frameworkService.showError(response.data);
                    }
                });
        };
        $scope.getToken();

        $scope.showPopup = function () {
            var text = 'Please read or display your payment token to the cashier when you are ready to pay.<br><br>';
            if ($scope.selected.cashback) {
                text += 'Funds will be deducted from your cashback balance first. ';
                if ($scope.selected.card) {
                    text += 'Any remaining funds will be deducted from the credit card selected.';
                } else {
                    text += 'Any remaining funds will be paid in store.';
                }
            } else {
                if ($scope.selected.card) {
                    text += 'Funds will be deducted from your credit card above.';
                } else {
                    text += 'Funds will be paid in store.';
                }
            }
            text += '<br><br>You will receive a payment receipt via email following the successful completion of a transaction.';

            frameworkService.showPopup({
                scope: $scope,
                template: i18nFilter(text),
                buttons: [{
                    text: i18nFilter('OK'),
                    type: 'button-balanced'
                }]
            });
        };
    };

    paymentTokenController.$inject = ['$scope', '$timeout', 'loyaltyService', 'frameworkService', 'paymentService', 'paymentMethodsResponse', 'i18nFilter', 'response_enum', 'utilityService'];
    return paymentTokenController;
});
