define([], function () {
    var reviewController = function ($scope, $stateParams, $window, $rootScope, frameworkService, reviewService, response_enum, reviewUtilService) {
        $scope.review = reviewUtilService.getReview();
        $scope.goReviewFields = reviewUtilService.getGoReviewFields();
        $scope.reviewResponse = $stateParams.reviewResponse;

        var starConfig = {
            iconOn: 'fas fa-star',
            iconOff: 'fas fa-star',
            iconOffColor: 'rgb(146,	146, 146)',
            rating: -1,
            minRating: -1
        };

        $scope.starsConfigReviewFields = _.extend({}, starConfig, {
            callback: function (rating, index) {
                var key = 'rating' + (index + 1);
                $scope.review[key] = rating;
            }
        });

        $scope.starsConfigAdditionalFields = _.extend({}, starConfig, {
            callback: function (rating, index) {
                var key = 'rating_extra_field' + (index + 1);
                $scope.review[key] = rating;
            }
        });

        $scope.optionSelected = function (field, value, index) {
            if (value === field.other_description) {
                delete $scope.review['rating_extra_field' + index];
                $scope.toggleGroup(index);
            } else {
                $scope.shownGroup = null;
            }
        };

        $scope.toggleGroup = function (key) {
            $scope.shownGroup = $scope.isGroupShown(key) ? null : key;
        };

        $scope.isGroupShown = function (key) {
            return $scope.shownGroup === key;
        };

        $scope.submitReview = function () {
            if (!_.every(Object.keys($scope.goReviewFields.rating_fields), function isPresent(field) {
                return field in $scope.review;
            }))
                return frameworkService.showAlert('Please provide a star rating for each field.');

            if ($scope.goReviewFields.rating_extra_fields) {
                let requiredKeys = [];
                _.map($scope.goReviewFields.rating_extra_fields, function (f, index) {
                    if (!f.optional) requiredKeys.push(index);
                });

                if (!_.every(requiredKeys, function isPresent(field) {
                    return field in $scope.review;
                }))
                    return frameworkService.showAlert('Please provide a response for each required field.');
            }

            var storeId = $scope.goReviewFields.location.clique_id;
            $scope.review.review_sub_section_name = $scope.goReviewFields.review_sub_section_name;

            reviewService.submitReview($scope.review, storeId).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    frameworkService.disableNextAnimation();
                    frameworkService.navigateTo('goReviewThanks', { reviewResponse: response.data });
                    reviewUtilService.clearReview();
                    reviewUtilService.clearGoReviewFields();
                } else if (_.some(response.data, { message: response_enum.ERROR_CODES.GOREVIEW_ALREADY_SUBMITTED })) {
                    frameworkService.showAlert('You have already submitted a review in the last 15 minutes.');
                } else if (_.some(response.data, { message: response_enum.ERROR_CODES.GOREVIEW_ERROR })) {
                    frameworkService.showAlert('GoReview error');
                } else {
                    frameworkService.showError(response.data);
                }
            });
        };

        $scope.openSocialPage = function (url) {
            frameworkService.openInBrowser(url);
        };

        $scope.shareInFacebook = function (url) {
            $window.plugins.socialsharing.shareViaFacebook('Message via Facebook', null, url, function () { }, function (err) { });
        };
    };

    reviewController.$inject = ['$scope', '$stateParams', '$window', '$rootScope', 'frameworkService', 'reviewService', 'response_enum', 'reviewUtilService'];
    return reviewController;
});
