define([], function () {
    var profileController = function ($scope, moment, userResponse, userService, frameworkService, config_constants, response_enum, paymentService) {
        $scope.user = userResponse.data;
        $scope.paymentMethod = $scope.user.payment;

        if (config_constants.loyalty.kids) {
            userService.getKids().then(function (response) {
                if (response.response_code == response_enum.CODE.SUCCESS) {
                    $scope.kids = response.data;
                    _.forEach($scope.kids, function (k) {
                        k.birthday = moment(k.birthday).format('DD/MM/YYYY');
                    });
                } else {
                    frameworkService.showError(response.data);
                }
            });
        }

        $scope.deletePaymentMethod = function () {
            paymentService.deletePaymentMethod().then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    frameworkService.showToast('Payment method deleted');
                    frameworkService.doRefresh();
                } else {
                    frameworkService.showError(response.data);
                }
            });
        };

        $scope.linkWithOAuth = function (provider) {
            userService.linkWithOAuth(provider)
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        frameworkService.showToast('Successfully linked');
                    } else {
                        var msg = _.get(_.head(response.data), 'message');
                        if (!_.isEmpty(msg)) {
                            if (msg === response_enum.ERROR_CODES.OAUTH_ALREADY_LINKED) {
                                frameworkService.showAlert('This account is already linked to another profile.');
                            } else {
                                frameworkService.showAlert(msg);
                            }
                        } else {
                            frameworkService.showError(response.data);
                        }
                    }
                });
        };

        $scope.unlinkFromOAuth = function (provider) {
            userService.unlinkFromOAuth(provider)
                .then(function (response) {
                    if (response.response_code === response_enum.CODE.SUCCESS) {
                        frameworkService.showToast('Successfully unlinked');
                    } else {
                        frameworkService.showError(response.data);
                    }
                });
        };
    };

    profileController.$inject = ['$scope', 'moment', 'userResponse', 'userService', 'frameworkService', 'config_constants', 'response_enum', 'paymentService'];
    return profileController;
});