define([], function () {
    var userActivateController = function ($scope, $stateParams, userService, frameworkService, response_enum, taggingService) {
        $scope.user = {
            username: $stateParams.username,
            token: null
        };

        $scope.triggerGtmEvent = function () {
            userService.getRegistrationDetails($scope.user.username).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    $scope.user.type = response.data.registration.type === 'physical' ? 'physicalRegistration' : 'digitalRegistration';

                    // add GTM event for registration type
                    var gtmEvent = $scope.user.type;
                    taggingService.pushEvent(gtmEvent, {});
                }
            });
        };

        $scope.resendToken = function () {
            userService.resendActivateUser($scope.user.username).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    frameworkService.showAlert('We have sent another SMS with a new token to your mobile.');
                } else if (_.some(response.data, { message: response_enum.ERROR_CODES.APPUSER_ALREADY_ACTIVED })) {
                    frameworkService.showAlert('Your account verification has already been completed.<br>Please proceed to login to the app.');
                } else {
                    frameworkService.showError(response.data);
                }
            });
        };

        $scope.submit = function () {
            return userService.activate($scope.user.token).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    $scope.triggerGtmEvent();

                    frameworkService.showAlert('Success! Your mobile has been validated.').then(function () {
                        frameworkService.navigateToHome({ tokenSuccess: true });
                    });
                } else if (_.some(response.data, { message: response_enum.ERROR_CODES.TOKEN_INVALID })) {
                    frameworkService.showAlert('Invalid token.');
                } else {
                    frameworkService.showError(response.data);
                }
            });
        };
    };

    userActivateController.$inject = ['$scope', '$stateParams', 'userService', 'frameworkService', 'response_enum', 'taggingService'];
    return userActivateController;
});