define([], function () {
    var userEditController = function ($scope, $stateParams, moment, userResponse, fencingService, frameworkService, keenService, userService, config_constants, response_enum) {
        $scope.selected = {
            page: _.toNumber(_.defaultTo($stateParams.tab, 0)),
            location: null,
            list: false
        };

        $scope.placeRestrictions = { country: fencingService.getCountry() };

        if (userResponse) {
            $scope.user = userResponse.data;
            $scope.user.birthday = moment($scope.user.birthday).format('DD/MM/YYYY');
            if ($scope.user.email === 'noemail@recipient.com.au') $scope.user.email = null;
            userService.getCliqueLocations().then(function (response) {
                $scope.locations = response.data;
                $scope.selected.location = _.get(_.find(response.data, { clique_id: $scope.user.favourite_location }), 'display_name');
            });
            $scope.isRegistering = false;
        } else {
            $scope.user = {
                first_name: null,
                last_name: null,
                email: null,
                mobile: null,
                birthday: null,
                card_number: null,
                gender: null,
                email_subscription: false,
                sms_subscription: false,
                password: null,
                favourite_food: null,
                favourite_location: null
            };
            $scope.isRegistering = true;
        }
        var originalMobile = $scope.user.mobile;
        var originalEmail = $scope.user.email;

        var showErrorAlert = function (response) {
            var msg;
            if (_.some(response.data, { message: response_enum.ERROR_CODES.APPUSER_EMAIL_ALREADY_IN_USE })) {
                msg = 'This email is already in use.';
            } else if (_.some(response.data, { message: response_enum.ERROR_CODES.APPUSER_MOBILE_ALREADY_IN_USE })) {
                msg = 'This mobile is already in use.';
            } else if (_.some(response.data, { message: response_enum.ERROR_CODES.MBM_USERNAME_ALREADY_IN_USE })) {
                msg = config_constants.username === 'mobile' ? 'This mobile is already in use.' : 'This email is already in use.';
            } else if (_.some(response.data, { message: response_enum.ERROR_CODES.CLIQUE_CARD_ALREADY_IN_USE })) {
                msg = 'This card is already in use.';
            } else if (_.some(response.data, { message: response_enum.ERROR_CODES.CLIQUE_CARD_INVALID })) {
                msg = 'This card is invalid.';
            } else if (_.some(response.data, { message: response_enum.ERROR_CODES.CLIQUE_CARD_NOT_FOUND })) {
                msg = 'This card could not be found.';
            } else {
                msg = _.get(_.head(response.data), 'message'); // Display the error msg even if it's not expected (Clique integration is unpredictable)
            }
            if (!_.isEmpty(msg)) {
                frameworkService.showAlert(msg);
            } else {
                frameworkService.showError(response.data);
            }
        };

        $scope.register = function () {
            if (!$scope.user.tc_checked) {
                frameworkService.showAlert('You need to accept the terms and conditions.');
                return;
            }
            var data = angular.copy($scope.user);
            data.username = config_constants.username === 'mobile' ? $scope.user.mobile : $scope.user.email;
            if (data.birthday) data.birthday = moment.utc(data.birthday, 'DD/MM/YYYY').toISOString();
            data.mobile_country = _.toUpper(data.mobile_country);

            userService.register(data).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    // add event for registration
                    keenService.recordEvent('registrations', response.data);

                    if (config_constants.username === 'mobile') {
                        frameworkService.showAlert('Thank you for joining.<br>Please check your phone to verify your account before login.').then(function () {
                            frameworkService.navigateTo('userActivate', { username: data.username });
                        });
                    } else {
                        frameworkService.showAlert('Thank you for joining.<br>Please check your email to verify your account before login.').then(function () {
                            frameworkService.navigateTo('login');
                        });
                    }
                } else {
                    showErrorAlert(response);
                }
            });
        };

        $scope.edit = function () {
            var data = angular.copy($scope.user);
            if (data.birthday) data.birthday = moment.utc(data.birthday, 'DD/MM/YYYY').toISOString();
            data.mobile_country = _.toUpper(data.mobile_country);
            data.username = config_constants.username === 'mobile' ? $scope.user.mobile : $scope.user.email;
            userService.update(data).then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    if (originalMobile !== data.mobile) {
                        frameworkService.showToast('An SMS has been sent to your new number with a token to verify it');
                    }
                    // if (originalEmail !== data.email) {
                    //     frameworkService.showToast('An email has been sent to your new address with a verification link');
                    // }
                    if ((originalMobile !== data.mobile && config_constants.username === 'mobile') ||
                        (originalEmail !== data.email && config_constants.username === 'email')) {
                        frameworkService.navigateTo('userUpdateUsername');
                    } else {
                        frameworkService.showToast('Your profile has been updated');
                        frameworkService.navigateTo('profile');
                    }
                } else {
                    showErrorAlert(response);
                }
            });
        };

        $scope.$watch('user.birthday', function () {
            var birthday = moment($scope.user.birthday, 'DD/MM/YYYY');
            (moment().diff(birthday, 'years') >= 18) ? $scope.age = moment().diff(birthday, 'years') : $scope.age = null;
        });

        $scope.onSelectLocation = function (location) {
            $scope.user.favourite_location = location.clique_id;
            $scope.selected.location = location.display_name;
            $scope.selected.list = false;
        };

        $scope.showList = function () {
            $scope.selected.list = true;
            if (_.isEmpty($scope.selected.location)) $scope.selected.list = false;
        };
    };

    userEditController.$inject = ['$scope', '$stateParams', 'moment', 'userResponse', 'fencingService', 'frameworkService', 'keenService', 'userService', 'config_constants', 'response_enum'];
    return userEditController;
});
