define([], function () {
    var advertFavourite = function ($rootScope, frameworkService, favouriteService, keenService, config_constants, response_enum) {
        return {
            restrict: 'E',
            replace: true,
            scope: {
                advert: '<'
            },
            templateUrl: 'directives/templates/advertFavourite.html',
            link: function ($scope, $element) {
                if (!$scope.advert) {
                    return;
                }

                $scope.favourite = {
                    document_id: $scope.advert._id,
                    type: 'advert',
                    is_favourite: false
                };

                var favourites = favouriteService.getFavourites();
                if (_.some(favourites, { document_id: $scope.advert._id })) {
                    $scope.favourite.is_favourite = true;
                }

                // Set a watcher because the user can favourite an advert instance, so the rest in the list should also update
                var onFavouriteUpdate = $rootScope.$on('favourite:update', function (event, favourite_document_id, is_favourite) {
                    if ($scope.favourite.document_id === favourite_document_id) {
                        // Save data on memory
                        $scope.favourite.is_favourite = is_favourite;
                    }
                });

                $scope.closePopover = function () {
                    $scope.popover.hide();
                };

                $scope.openPopover = function ($event) {
                    $event.preventDefault();
                    $event.stopPropagation();

                    frameworkService.createPopover($scope, 'views/other/favouritePopover.html', 'slide-in-up')
                        .then(function (popover) {
                            $scope.objectsToUpdate = [{
                                favourite: $scope.favourite,
                                name: $scope.advert.title,
                                makeFavourite: true
                            }];
                            favourites = favouriteService.getFavourites();

                            if (!config_constants.isSingleMerchant) {
                                if (!_.some(favourites, { document_id: $scope.advert.merchant._id })) {
                                    $scope.objectsToUpdate.push({
                                        favourite: {
                                            document_id: $scope.advert.merchant._id,
                                            type: 'merchant',
                                            is_favourite: false
                                        },
                                        name: $scope.advert.merchant.display_name,
                                        makeFavourite: true
                                    });
                                }
                            }
                            if (!_.some(favourites, { document_id: $scope.advert.location._id })) {
                                $scope.objectsToUpdate.push({
                                    favourite: {
                                        document_id: $scope.advert.location._id,
                                        type: 'location',
                                        is_favourite: false
                                    },
                                    name: $scope.advert.location.display_name,
                                    makeFavourite: true
                                });
                            }

                            _.forEach($scope.advert.categories, function (category) {
                                if (!_.some(favourites, { document_id: category._id })) {
                                    $scope.objectsToUpdate.push({
                                        favourite: {
                                            document_id: category._id,
                                            type: 'category',
                                            is_favourite: false
                                        },
                                        name: category.name,
                                        makeFavourite: true
                                    });
                                }
                            });

                            $scope.popover = popover;
                            var stopListening = $scope.$on('popover.hidden', function () {
                                stopListening();
                                popover.remove();
                            });
                            $scope.popover.show($event);
                        });
                };

                $scope.unfavourite = function ($event) {
                    $event.preventDefault();
                    $event.stopPropagation();

                    favouriteService.delete($scope.favourite, $scope.advert.merchant._id)
                        .then(function (response) {
                            if (response.response_code === response_enum.CODE.SUCCESS) {
                                keenService.recordEvent('favourites_remove', {
                                    favourite: $scope.favourite,
                                    merchant_id: $scope.advert.merchant._id
                                });
                                $scope.favourite.is_favourite = false;
                                $rootScope.$emit('favourite:update', $scope.favourite.document_id, $scope.favourite.is_favourite);
                                frameworkService.showToast('Favourite deleted');
                            } else {
                                frameworkService.showError(response.data);
                            }
                        });
                };

                $scope.save = function () {
                    _.forEach($scope.objectsToUpdate, function (o) {
                        if (o.makeFavourite) {
                            favouriteService.save(o.favourite, $scope.advert.merchant._id)
                                .then(function (response) {
                                    if (response.response_code === response_enum.CODE.SUCCESS) {
                                        keenService.recordEvent('favourites_add', {
                                            favourite: o.favourite,
                                            merchant_id: $scope.advert.merchant._id
                                        });
                                        if ($scope.favourite.document_id === o.favourite.document_id) {
                                            $scope.favourite.is_favourite = true;
                                            $rootScope.$emit('favourite:update', $scope.favourite.document_id, $scope.favourite.is_favourite);
                                        } else {
                                            _.pull($scope.objectsToUpdate, o);
                                        }
                                    }
                                });
                        }
                    });
                    if (!_.isEmpty($scope.objectsToUpdate)) {
                        frameworkService.showToast('Favourites added');
                    }
                };

                $scope.$on('$destroy', function () {
                    onFavouriteUpdate();
                    if ($scope.popover) {
                        $scope.popover.remove();
                    }
                });
            }
        };
    };

    advertFavourite.$inject = ['$rootScope', 'frameworkService', 'favouriteService', 'keenService', 'config_constants', 'response_enum'];
    return advertFavourite;
});