define([], function () {
    var favourite = function (frameworkService, favouriteService, keenService, response_enum) {
        return {
            restrict: 'E',
            replace: true,
            scope: {
                type: '@',
                id: '@',
                merchantId: '@' // Send it to the server for reports
            },
            templateUrl: 'directives/templates/favourite.html',
            link: function ($scope, $element) {
                var favourites = favouriteService.getFavourites();

                $scope.favourite = _.find(favourites, { document_id: $scope.id });
                if ($scope.favourite) {
                    $scope.favourite.is_favourite = true;
                } else {
                    $scope.favourite = {
                        document_id: $scope.id,
                        type: $scope.type,
                        is_favourite: false
                    };
                }

                $scope.editFavourite = function (event) {
                    event.preventDefault();
                    event.stopPropagation();

                    $scope.data = angular.copy($scope.favourite);

                    if ($scope.data.is_favourite) {
                        favouriteService.delete($scope.data)
                            .then(function (response) {
                                if (response.response_code == response_enum.CODE.SUCCESS) {
                                    keenService.recordEvent('favourites_remove', {
                                        favourite: $scope.data,
                                        merchant_id: $scope.merchantId
                                    });
                                    // Save data on memory
                                    $scope.favourite.is_favourite = false;
                                    frameworkService.showToast('Favourite removed');
                                } else {
                                    frameworkService.showError(response.data);
                                }
                            });
                    } else {
                        favouriteService.save($scope.data)
                            .then(function (response) {
                                if (response.response_code == response_enum.CODE.SUCCESS) {
                                    keenService.recordEvent('favourites_add', {
                                        favourite: $scope.data,
                                        merchant_id: $scope.merchantId
                                    });
                                    // Save data on memory
                                    $scope.favourite.is_favourite = true;
                                    frameworkService.showToast('Favourite added');
                                } else {
                                    frameworkService.showError(response.data);
                                }
                            });
                    }
                };
            }
        };
    };

    favourite.$inject = ['frameworkService', 'favouriteService', 'keenService', 'response_enum'];
    return favourite;
});
