define([], function () {
    var onRun = function ($rootScope, $window, $location, tmhDynamicLocale, fencingService, frameworkService, geolocationService, notificationService, userService, config_constants, country_options, response_enum, taggingService, leftMenuService, loyaltyService) {
        if ($window.cordova) {
            navigator.splashscreen.hide();
        }

        var counter = frameworkService.appStartedCounter();
        frameworkService.initBranch();
        frameworkService.registerBackButtonAction();
        // frameworkService.promptForReview();
        frameworkService.promptWalkthrough();

        leftMenuService.removeButton('categoryList');

        var stateWatcher = $rootScope.$on('$stateChangeStart', function (event, state) {
            var statesWithoutGeolocation = [
                'userActivate',
                'resetPassword',
                'userUpdateUsername'
            ];
            if (_.every(statesWithoutGeolocation, function (item) { return item !== state.name; })) {
                // Get geolocation on run, but avoid asking for it for certain routes that do not need it
                stateWatcher();
                geolocationService.getCurrentGeolocation();
            }
        });

        // Google Tag Manager. PageView tracking
        $rootScope.$on('$stateChangeSuccess', function () {
            taggingService.pushEvent('pageView', { 'action': $location.path() });
        });

        if (config_constants.international) {
            fencingService.guessCountry()
                .then(function (result) {
                    var changeCountry = function () {
                        fencingService.setCountry(result.country_code);
                        frameworkService.doRefresh();
                    };
                    if (!_.find(country_options, { _id: result.country_code }) || fencingService.getCountry() === result.country_code) return;

                    if (counter > 1) {
                        frameworkService.showConfirm(
                            'New location detected',
                            'Welcome to ' + result.country_name + '. While you are here, we will change your country, so the content is appropiate. You can always change it back from Settings.',
                            changeCountry
                        );
                    } else {
                        changeCountry();
                    }
                });
        }

        tmhDynamicLocale.set(fencingService.getLocale());

        frameworkService.showLoading(); // Wait until the UUID is set to start using the app
        try {
            // Initialize notification service and set user's UUID
            notificationService.init();
        } catch (err) {
            if (!userService.getUUID()) {
                // Ensure a UUID value for Web and Ionic View testing
                var uuid = 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
                    // Some old browsers doesn't have crypto
                    var r = ($window.crypto && $window.crypto.getRandomValues) ? $window.crypto.getRandomValues(new Uint8Array(1))[0] % 16 | 0 : Math.random() * 16 | 0;
                    var v = c == 'x' ? r : (r & 0x3 | 0x8);
                    return v.toString(16);
                });
                userService.setUUID(uuid);
            }
        } finally {
            frameworkService.hideLoading();
        }

        // Get user info
        if (userService.isAuthenticated()) {
            userService.getLoggedInUser().then(function (response) {
                if (response.response_code !== response_enum.CODE.SUCCESS) {
                    userService.logout();
                }
            });

            loyaltyService.getLoyaltyDetails().then(function (response) {
                if (response.response_code === response_enum.CODE.SUCCESS) {
                    var favourites = _.pick(response.data, ['favourite_food', 'favourite_location']);
                    var emptyFields = [];
                    _.findKey(favourites, function (item) {
                        if (item === null) emptyFields.push(item);
                    })
                    if (!_.isEmpty(emptyFields) && localStorageService.retrieve('startCounter') % 5 === 0) {
                        frameworkService.showConfirm('COMPLETE PROFILE', 'Press OK to complete all your profile information.', function () {
                            frameworkService.navigateTo('userEdit', { tab: 1 });
                        });
                    }
                }
            });
        }
    };

    onRun.$inject = ['$rootScope', '$window', '$location', 'tmhDynamicLocale', 'fencingService', 'frameworkService', 'geolocationService', 'notificationService', 'userService', 'config_constants', 'country_options', 'response_enum', 'taggingService', 'leftMenuService', 'loyaltyService'];
    return onRun;
});
