define([], function () {
    var userEvents = function ($rootScope, userService, favouriteService, frameworkService, keenService, i18nFilter, config_constants, response_enum) {
        var showReferralPopup = function (user) {
            if (!_.isUndefined(user.referral.referred_by)) return;

            var $scope = $rootScope.$new();
            $scope.referral = { code: null };
            frameworkService.showPopup({
                templateUrl: 'views/user/referralPopup.html',
                title: i18nFilter('Have a referral code?'),
                scope: $scope,
                buttons: [{
                    text: i18nFilter('Skip'),
                    type: 'button-assertive',
                    onTap: function ($event) {
                        userService.saveReferralCode(null);
                    }
                }, {
                    text: i18nFilter('Submit'),
                    type: 'button-balanced',
                    onTap: function ($event) {
                        if (!$scope.referral.code) {
                            if ($event) $event.preventDefault();
                            return;
                        }
                        userService.saveReferralCode($scope.referral.code)
                            .then(function (response) {
                                if (response.response_code === response_enum.CODE.SUCCESS) {
                                    frameworkService.showToast('The referral code has been saved.');
                                    keenService.recordEvent('referrals_use', { code: $scope.referral.code });
                                } else {
                                    if (_.some(response.data, { message: response_enum.ERROR_CODES.APPUSER_NOT_FOUND })) {
                                        frameworkService.showAlert('Incorrect referral code.')
                                            .then(function () {
                                                showReferralPopup(user);
                                            });
                                    } else {
                                        frameworkService.showError(response.data);
                                    }
                                }
                            });
                    }
                }]
            });
        }

        $rootScope.$on('user:login', function (event, user) {
            favouriteService.setFavourites(user.favourites);
            if (_.get(config_constants, 'loyalty.referral')) {
                showReferralPopup(user);
            }
        });

        $rootScope.$on('user:logout', function () {
            favouriteService.clearFavourites();
        });

    };

    userEvents.$inject = ['$rootScope', 'userService', 'favouriteService', 'frameworkService', 'keenService', 'i18nFilter', 'config_constants', 'response_enum'];
    return userEvents;
});
