define([], function () {
    var favouriteService = function (restService, localStorageService, response_enum) {
        var self = this;

        this.getFavourites = function () {
            return localStorageService.retrieve('favourites') || [];
        };

        this.setFavourites = function (data) {
            localStorageService.store('favourites', data);
        };

        this.clearFavourites = function () {
            localStorageService.remove('favourites');
        };

        this.updateFavouritesFromServer = function () {
            return restService.get('/favourite')
                .then(function (response) {
                    if (response.response_code == response_enum.CODE.SUCCESS) {
                        self.setFavourites(response.data);
                    }
                    return response;
                });
        };

        this.save = function (favourite) {
            var saveInLocalStorage = function (response) {
                if (response.response_code == response_enum.CODE.SUCCESS || _.some(response.data, { message: response_enum.ERROR_CODES.FAVOURITE_OBJECT_ALREADY_FAVOURITED })) {
                    var local_favourites = self.getFavourites();
                    if (favourite.is_favourite) {
                        _.remove(local_favourites, { document_id: favourite.document_id });
                    }
                    local_favourites.push(favourite);
                    self.setFavourites(local_favourites);
                }
                return response;
            };

            var data = {
                document_id: favourite.document_id,
                type: favourite.type
            };
            // Save in the server
            return restService.post('/favourite', data).then(saveInLocalStorage);
        };

        this.delete = function (favourite) {
            var removeInLocalStorage = function (response) {
                if (response.response_code == response_enum.CODE.SUCCESS || _.some(response.data, { message: response_enum.ERROR_CODES.FAVOURITE_OBJECT_NOT_FAVOURITED })) {
                    var local_favourites = self.getFavourites();
                    _.remove(local_favourites, { document_id: favourite.document_id });
                    self.setFavourites(local_favourites);
                }
                return response;
            };
            // Remove from the server
            return restService.delete('/favourite/' + favourite.document_id).then(removeInLocalStorage);
        };
    };

    favouriteService.$inject = ['restService', 'localStorageService', 'response_enum'];
    return favouriteService;
});
