define([], function () {
    var frameworkService = function ($auth, $rootScope, $window, $state, $timeout, $ionicPopup, $ionicPlatform, $ionicActionSheet, $ionicModal, $ionicPopover, $ionicLoading, $ionicSideMenuDelegate, $ionicScrollDelegate, $ionicHistory, ionicDatePicker, moment, localStorageService, surveyService, config_constants, response_enum, i18nFilter) {
        var self = this;

        this.openInBrowser = function (url) {
            $window.open(url, '_system');
        };

        this.openInModalIFrame = function (title, url) { };

        this.getHomeState = function () {
            return _.get(config_constants, 'homeState.mobile.' + ($auth.isAuthenticated() ? 'user' : 'guest'), 'home');
        };

        this.navigateTo = function (state, params, options) {
            return $state.go(state, params, options);
        };

        this.navigateToHome = function (params) {
            self.disableNextAnimation();
            return self.navigateTo(self.getHomeState(), params);
        };

        this.navigateToNewOrder = function (menu_type) {
            var params = { menu_type: _.defaultTo(menu_type, 'standard') };
            if (config_constants.ordering.flow === 'menus_first') {
                return self.navigateTo('menuList', params);
            } else {
                return self.navigateTo('orderLocationList', params);
            }
        };

        var wasStateWithTabs;
        var isStateWithTabs;
        this.onStateChangeStart = function (event, state) {
            wasStateWithTabs = isStateWithTabs;
            isStateWithTabs = _.includes(['favouriteList', 'vouchercodeList'], state.parent);
        };

        // Fixes Ionic bug that does not show the back button after transitioning from a tab view
        this.showBackButton = function ($scope) {
            if (wasStateWithTabs) {
                $scope.$on('$ionicView.beforeEnter', function (event, state) {
                    state.enableBack = true;
                });
            }
        };

        this.disableNextAnimation = function () {
            isStateWithTabs = false; // use case: user is in the tabs page (favourites or vouchers) and goes to the advert list from the left menu
            $ionicHistory.nextViewOptions({
                disableAnimate: true,
                historyRoot: true
            });
        };

        this.onGeolocationDisabled = function () {
            self.showAlert('Please enable location services on your device.');
        };

        this.appStartedCounter = function () {
            var counter = _.add(localStorageService.retrieve('startCounter'), 1);
            localStorageService.store('startCounter', counter);
            return counter;
        };

        this.promptForReview = function () {
            if (localStorageService.retrieve('hasRated')) {
                return;
            }
            var counter = localStorageService.retrieve('startCounter');
            var ratingPromptCount = _.defaultTo(config_constants.ratingPromptCount, 3);
            if (counter % ratingPromptCount === 0) {
                $timeout(function () {
                    self.showPopup({
                        title: i18nFilter('Do you love ') + ' ' + i18nFilter('App') + '?',
                        templateUrl: 'views/other/ratingPopup.html',
                        buttons: [{
                            text: i18nFilter('Later'),
                            type: 'button-assertive'
                        }, {
                            text: i18nFilter('Rate now'),
                            type: 'button-balanced',
                            onTap: function (e) {
                                if (self.getPlatform() == 'android') {
                                    $window.open('market://details?id=' + config_constants.marketId.android, '_system');
                                } else if (self.getPlatform() == 'ios') {
                                    $window.open('itms-apps://itunes.apple.com/app/id' + config_constants.marketId.ios, '_system');
                                } else {
                                    self.showAlert('This feature is available only in the app.');
                                }
                                localStorageService.store('hasRated', true);
                            }
                        }]
                    });
                }, 3000);
            }
        };

        this.promptWalkthrough = function () {
            if (!config_constants.walkthrough || localStorageService.retrieve('hasViewedWalkthrough') || !$window.cordova) {
                return;
            }
            localStorageService.store('hasViewedWalkthrough', true);
            var $scope = $rootScope.$new();
            self.showModal($scope, 'views/other/walkthrough.html');
        };

        this.promptSurvey = function (user) {
            surveyService.getSurvey()
                .then(function (response) {
                    if (_.isNil(_.get(response, 'data'))) return;

                    var $scope = $rootScope.$new();
                    $scope.survey = response.data;
                    $scope.submit = function () {
                        surveyService.saveSurvey($scope.survey)
                            .then(function () {
                                self.doRefresh();
                                $scope.ionModal.remove();
                                self.showToast('Thank you');
                            });
                    };
                    self.showModal($scope, 'views/user/surveyModal.html');
                });
        };

        this.goBack = function () {
            $ionicHistory.goBack();
        };

        this.showAlert = function (message) {
            return $ionicPopup.alert({
                title: i18nFilter(message)
            });
        };

        this.showError = function (error) {
            console.error(error);
            if (_.some(error, { message: response_enum.ERROR_CODES.LOCATION_OFFLINE })) {
                self.showAlert('Sorry, this restaurant is offline, please try again later.');
            } else if (_.some(error, { message: response_enum.ERROR_CODES.CLIQUE_CARD_DISABLED })) {
                self.showAlert('Your loyalty card is disabled.');
            } else {
                self.showAlert('Sorry, something went wrong. We are working on it and we will get it fixed as soon as we can.');
            }
        };

        this.scrollTop = function (offset) {
            if (offset) {
                $ionicScrollDelegate.scrollBy(0, offset);
            } else {
                $ionicScrollDelegate.scrollTop();
            }
        };

        this.resizeScrollableWindow = function () {
            $timeout(function () {
                $ionicScrollDelegate.resize();
            }, 1000);
        };

        this.showToast = function (message) {
            if ($window.plugins && $window.plugins.toast) {
                $window.plugins.toast.showWithOptions({
                    message: i18nFilter(message),
                    duration: 'short',
                    position: 'bottom',
                    addPixelsY: -40 // added a negative value to move it up a bit (default 0)
                });
            } else {
                console.log('TOAST:', message);
            }
        };

        this.toggleLeftMenu = function () {
            $ionicSideMenuDelegate.toggleLeft();
        };

        this.showModal = function ($scope, templateUrl) {
            return $ionicModal.fromTemplateUrl(templateUrl, {
                scope: $scope
            }).then(function (modal) {
                $scope.ionModal = modal;
                modal.show();
            });
        };

        this.createPopover = function ($scope, templateUrl, animation) {
            return $ionicPopover.fromTemplateUrl(templateUrl, {
                scope: $scope,
                animation: animation
            });
        };

        this.showPopup = function (options) {
            return $ionicPopup.show(options);
        };

        this.showConfirm = function (title, message, onTap) {
            return $ionicPopup.confirm({
                title: i18nFilter(title),
                template: i18nFilter(message),
                buttons: [{
                    text: i18nFilter('Cancel'),
                    type: 'button-assertive'
                }, {
                    text: i18nFilter('OK'),
                    type: 'button-balanced',
                    onTap: onTap
                }]
            });
        };

        this.showActionSheet = function (options) {
            return $ionicActionSheet.show(options);
        };

        this.doRefresh = function () {
            $state.reload();
            $rootScope.$broadcast('scroll.refreshComplete');
        };

        this.getDatepickerOptions = function (options) {
            var config = {
                inputDate: moment(options.date).toDate(),
                from: moment(options.start).toDate(),
                to: moment(options.end).toDate(),
                mondayFirst: false,
                closeOnSelect: true,
                templateType: 'popup',
                disabledDates: _.map(options.disabled, function (d) {
                    return moment(d).toDate();
                })
            };

            config.open = function () { ionicDatePicker.openDatePicker(config); };
            config.callback = function (value) {
                var justDate = moment(value).format('YYYY-MM-DD'); // Format to date to then set the right TZ
                config.inputDate = moment.tz(justDate, 'YYYY-MM-DD', options.timezone).toDate();
                if (_.isFunction(options.callback)) options.callback(config.inputDate);
            };
            return config;
        };

        var loadingCounter = 0;
        var toggleLoading = function () {
            if (loadingCounter > 0) {
                $ionicLoading.show();
            } else {
                $ionicLoading.hide();
            }
        };
        this.showLoading = function () {
            loadingCounter++;
            toggleLoading();
        };
        this.hideLoading = function () {
            loadingCounter--;
            toggleLoading();
        };

        var pressBackButtonToExit = false;
        this.registerBackButtonAction = function () {
            $ionicPlatform.registerBackButtonAction(function () { // Back button handler for Android
                if (pressBackButtonToExit) {
                    ionic.Platform.exitApp();
                } else if ($state.is('landing') || $state.is(self.getHomeState())) {
                    pressBackButtonToExit = true;
                    self.showToast('Press back button again to exit');
                    $timeout(function () {
                        pressBackButtonToExit = false;
                    }, 2000);
                } else {
                    self.goBack();
                }
            }, 101); // overrides the ‘return to previous view’ action
        };

        this.loadMoreData = function ($scope, fetcher) {
            return fetcher.next().then(function (response) {
                if (_.isEmpty(response)) $scope.moreDataCanBeLoaded = false;
                $scope.$broadcast('scroll.infiniteScrollComplete'); // Used for mobile ionInfiniteScroll
                return response;
            });
        };

        this.initBranch = function () {
            if (!$window.cordova || !$window.Branch) return;
            var callback = function () {
                Branch.initSession().then(function (data) {
                    console.log('Deep Link data: ', angular.toJson(data));
                    if (data.state) {
                        self.navigateTo(data.state, angular.fromJson(data.params), {
                            reload: true
                        });
                    }
                });
            };
            $ionicPlatform.on('deviceready', callback);
            $ionicPlatform.on('resume', callback);
        };

        var platform = ionic.Platform.platform();
        if (!_.includes(['ios', 'android'], platform))
            platform = 'web';
        this.getPlatform = function () {
            return platform;
        };
    };

    frameworkService.$inject = ['$auth', '$rootScope', '$window', '$state', '$timeout', '$ionicPopup', '$ionicPlatform', '$ionicActionSheet', '$ionicModal', '$ionicPopover', '$ionicLoading', '$ionicSideMenuDelegate', '$ionicScrollDelegate', '$ionicHistory', 'ionicDatePicker', 'moment', 'localStorageService', 'surveyService', 'config_constants', 'response_enum', 'i18nFilter'];
    return frameworkService;
});
