define([], function () {
    var giftcardUtilService = function ($rootScope, localStorageService) {
        // Save it in memory as a singleton so it can be shared in more than one controller at a time
        var giftcards = localStorageService.retrieve('giftcards') || {};
        this.getGiftcards = function () {
            return giftcards;
        };

        this.setGiftcards = function (data) {
            giftcards = data;
            localStorageService.store('giftcards', data);
        };

        this.clearGiftcards = function () {
            _.forEach(giftcards, function (value, key) {
                giftcards[key] = undefined;
            });
            localStorageService.remove('giftcards');
        };

        this.getTotalCart = function () {
            return Math.max(_.sumBy(giftcards.recipients, 'amount'), 0);
        };

        this.getTotalNet = function () {
            return Math.max(this.getTotalCart(giftcards) + _.sumBy(giftcards.pre_total_net, function (m) {
                return m.amount || m.amountFn();
            }), 0);
        };

        this.calculateDiscount = function (type, price, discount) {
            if (type === 'value')
                return Math.min(price, discount);
            else if (type === 'percentage')
                return Math.min(price, price * discount / 100);
        };

        this.calculateCartDiscount = function (discount) {
            return _.sumBy(giftcards.recipients, (function (giftcard) {
                return _.includes(discount.giftcards, giftcard.original_amount) ? this.calculateDiscount(discount.type, giftcard.original_amount, discount.amount) : 0;
            }).bind(this));
        };

        this.addPreTotalNet = function (giftcards, modifier) {
            if (!giftcards.pre_total_net) {
                giftcards.pre_total_net = [];
            }
            giftcards.pre_total_net = _.concat(giftcards.pre_total_net, modifier);
        };

        this.removePreTotalNetByType = function (giftcards, type) {
            _.remove(giftcards.pre_total_net, { type: type });
        };

        this.clearPreTotalNet = function (giftcards) {
            giftcards.pre_total_net = [];
        };

        $rootScope.$on('user:logout', (function () {
            this.clearGiftcards();
        }).bind(this));
    };

    giftcardUtilService.$inject = ['$rootScope', 'localStorageService'];
    return giftcardUtilService;
});
