define([], function () {
    var leftMenuService = function ($rootScope, config_constants, i18nFilter) {
        var self = this;
        var menu = [];

        self.addButton = function (button) {
            self.removeButton(button.state);
            menu.push(button);
        };

        self.setButtonVisibility = function (state, visibility) {
            var item = _.find(menu, { state: state });
            if (item) {
                item.hidden = !visibility;
            }
        };

        self.disableButton = function (state, disability) {
            var item = _.find(menu, { state: state });
            if (item) {
                item.disabled = !disability;
            }
        };

        self.addChild = function (state, button) {
            var item = _.find(menu, { state: state });
            if (item) {
                item.children = _.castArray(item.children);
                _.remove(item.children, { state: button.state });
                item.children.push(button);
            }
        };

        self.removeButton = function (state) {
            _.remove(menu, { state: state });
        };

        self.changeSort = function (state, sort) {
            var item = _.find(menu, { state: state });
            if (item) {
                item.sort = sort;
            }
        };

        self.addButton({
            name: i18nFilter('My account'),
            icon: 'fa-user',
            state: 'myAccount',
            sort: 4,
            children: [{
                name: i18nFilter('My profile'),
                icon: 'fa-id-card',
                state: 'profile',
                sort: 1
            }, {
                name: i18nFilter('My coupons'),
                icon: 'fa-ticket-alt',
                state: 'vouchercodeIssuedList',
                sort: 5
            }, {
                name: i18nFilter('My favourites'),
                icon: 'fa-star',
                state: 'favouriteAdvertList',
                sort: 25
            }]
        });

        self.addButton({
            name: i18nFilter('Adverts'),
            icon: 'icomoon icomoon-advert',
            state: 'advertList',
            sort: 7
        });

        self.addButton({
            name: i18nFilter('Merchants'),
            icon: 'fa-building',
            state: 'merchantList',
            hidden: config_constants.isSingleMerchant,
            sort: 10
        });

        self.addButton({
            name: i18nFilter('Find a Restaurant'),
            icon: 'fa-map-marker-alt',
            state: 'locationList',
            sort: 15
        });

        self.addButton({
            name: !config_constants.ordering.readonly ? i18nFilter('Order online') : i18nFilter('Menu'),
            icon: 'icomoon icomoon-menu',
            state: _.get(config_constants, 'ordering.flow') == 'menus_first' ? 'menuList' : 'orderLocationList',
            // hidden: !config_constants.ordering,
            hidden: config_constants.ordering.hide,
            badge: !config_constants.ordering.readonly ? 'new' : null,    // do not show the new badge for other white labels that do not have ordering on atm
            sort: !config_constants.ordering.readonly ? 5 : 29   // put position in usual place when ordering as readonly, higher up if ordering turned on 
        });

        // This button will be replaced for a proper advert filter in the advertList
        self.addButton({
            name: i18nFilter('Categories'),
            icon: 'fa-th-large',
            state: 'categoryList',
            sort: 26
        });

        self.addButton({
            name: i18nFilter('Pizza bust'),
            icon: 'icomoon icomoon-pizza-bust',
            webUrl: config_constants.social.pizza_bust,
            hidden: !config_constants.other.pizza_bust,
            sort: 30
        });

        self.addButton({
            name: i18nFilter('Sign up'),
            icon: 'fa-user-plus',
            state: 'register',
            sort: 48
        });

        self.addButton({
            name: i18nFilter('Login'),
            icon: 'fa-sign-in',
            state: 'login',
            sort: 49
        });

        self.addButton({
            name: i18nFilter('Logout'),
            icon: 'fa-sign-out',
            state: 'logout',
            sort: 50
        });

        self.getMenu = function () {
            return menu;
        };

        $rootScope.$watch('isLoggedIn', function () {
            self.setButtonVisibility('myAccount', $rootScope.isLoggedIn);
            self.setButtonVisibility('logout', $rootScope.isLoggedIn);
            self.setButtonVisibility('register', !$rootScope.isLoggedIn);
            self.setButtonVisibility('login', !$rootScope.isLoggedIn);
            self.setButtonVisibility('kidsBenefits', (!$rootScope.isLoggedIn && config_constants.loyalty.kids));
        });

    };
    leftMenuService.$inject = ['$rootScope', 'config_constants', 'i18nFilter'];
    return leftMenuService;
});
