define(['stripe'], function () {
    var paymentService = function (restService, frameworkService, config_constants, userService) {
        var stripe = Stripe(config_constants.stripeKey);
        var elements = stripe.elements({
            fonts: [{
                family: 'Open Sans',
                src: 'local("Open Sans"), local("OpenSans"), url(https://fonts.gstatic.com/s/opensans/v13/cJZKeOuBrn4kERxqtaUH3ZBw1xU1rKptJj_0jans920.woff2) format("woff2")',
            }]
        });
        var card = elements.create('card', {
            hidePostalCode: true,
            style: {
                base: {
                    color: '#555',
                    fontFamily: 'Open Sans',
                    fontSize: '16px',
                    lineHeight: '48px',
                    fontSmoothing: 'antialiased',
                    '::placeholder': {
                        color: '#C1C1C1'
                    }
                }
            }
        });

        this.createStripeForm = function (selector) {
            card.mount(selector);
            return card;
        };

        this.createStripeToken = function (card) {
            frameworkService.showLoading();
            return stripe.createToken(card).then(function (result) {
                frameworkService.hideLoading();
                return result;
            });
        };

        this.generatePaymentToken = function (data) {
            return restService.post('/payment/token', data);
        };

        this.addPaymentMethod = function (stripe_token) {
            var url = '/user/payment/method';
            var data = {
                stripe_token: stripe_token
            };
            return restService.post(url, data);
        };

        this.deletePaymentMethod = function () {
            return restService.delete('/user/payment/method');
        };

        // below is from the oneScan paymentService file. since oneScan is not being enabled for the other apps in the Gruntfile 
        // it was not overriding this base file and thus not allowing the listPyamentMethod to be resolved as was using stripe routes
        // TODO: change base paymentService file to accommodate VCS
        this.listPaymentMethods = function () {
            return userService.getLoggedInUser().then(function (response) {
                return {
                    response_code: response.response_code,
                    data: response.data.payment ? [response.data.payment] : []
                };
            });
        };

        this.getPaymentById = function (_id) {
            return restService.get('/payment/' + _id);
        };

        this.authorisePayment = function (data) {
            var apiUrl = '/external/clique/order/payment/authorisation';
            return restService.post(apiUrl, data);
        };

        this.getVoucherDetails = function (data) {
            var apiUrl = '/external/clique/voucher/voucherdetails';
            return restService.post(apiUrl, data);
        };
    };

    paymentService.$inject = ['restService', 'frameworkService', 'config_constants', 'userService'];
    return paymentService;
});